@@ -27,6 +27,27 @@
 /* needed to see if WITH_SPC is defined */
 #include "config.h"
 
+#define WITH_MSG_QUEUE
+
+#ifdef WITH_MSG_QUEUE
+#define BLUEFISH_MSG_QUEUE 9723475
+#define MSQ_QUEUE_SIZE 1024
+#define MSQ_QUEUE_SMALL_SIZE 7
+#define MSQ_QUEUE_CHECK_TIME 300
+#define MSG_QUEUE_SEND_ALIVE 46064
+#define MSG_QUEUE_OPENFILE 46063
+#define MSG_QUEUE_ASK_ALIVE 46062
+
+#endif
+
+/* the saved selection in a document */
+typedef struct {
+	gboolean saved;
+	guint start_pos;
+	guint end_pos;
+	guint has_selection : 1;
+} TSelection;
+
 /* This is the type of each document */
 typedef struct {
 	gchar *filename;
@@ -41,6 +62,7 @@ typedef struct {
 	GtkWidget *tab_label;
 	gint highlightstate;
 	gint need_highlighting;		/* +1 means a '"' is added, +2 means a '<' is added, 4 means a '>' is added */
+	TSelection saved_sel;
 } Tdocument;
 
 /* This is the current project */
@@ -56,7 +78,7 @@ typedef struct {
 	GList *colorlist;
 	GList *dtd_cblist;
 	GList *classlist;
-	GList *metalist;
+	GList *headerlist;
 	GList *filenames;
 } Tproject;
 
@@ -64,8 +86,19 @@ typedef struct {
 	gint v_html_tb;				/* view html toolbar */
 	gint v_custom_tb;			/* view custom menubar */
 	gint v_main_tb;				/* view main toolbar */
+	gint transient_htdialogs;  /* set html dialogs transient ro the main window */
 	gint main_window_h;			/* main window height */
 	gint main_window_w;			/* main window width */
+	gint main_filelist_size; 	/* width of filelist */
+	gint max_recent_files;	/* length of Open Recent list */
+	gint max_dir_history;	/* length of directory history */
+	gchar *filelist_filter;	/* filelist filter type */
+	gchar *filebrowser_filter_image; /* the image type */
+	gchar *filebrowser_filter_php; /* the php type */
+	gchar *filebrowser_filter_html; /* the html type */
+	gchar *filebrowser_filter_jscript; /* the jscript type */
+	gchar *filebrowser_filter_css; /* the css type */
+	gchar *filebrowser_filter_other; /* the other type */
 	gchar *cfg_browser_cline;	/* browser commandline */
 	gchar *cfg_weblint_cline;	/* weblint command line */
 	gchar *cfg_editor_font;		/* editor font */
@@ -76,18 +109,33 @@ typedef struct {
 	gchar *cfg_thumbnailtype;	/* fileformat to use for thumbnails */
 	gint full_p;				/* use </p> */
 	gint full_li;				/* use </li> */
+	gint allow_css;				/* CSS allowed */
+	gint allow_dep;				/* allow <FONT>... */
+	gint format_by_context; 	/* use <strong> instead of <b>, <emphasis instead of <i> etc. (W3C reccomendation) */
+	gint xhtml;					/* write <br /> */
+	gint allow_ruby;			/* allow <ruby> */
+	gint allow_h4;				/* allow <Q>... */
+	gint allow_frames;			/* allow <FRAME> */
+	gint force_dtd;				/* write <!DOCTYPE...> */
+	gint dtd_url;				/* URL in DTD */
+	gint xml_start;				/* <?XML...> */
 	gint lowercase_tags;		/* use lowercase tags */
 	gint word_wrap;				/* use wordwrap */
+	gint autoindent;			/* autoindent code */
 	gint line_wrap;				/* use line wrap */
 	gint fontset;				/* load fontset (0 = load font) */
+	gint force_def_style;  /* force white editor background */
+	gint drop_at_drop_pos; 	/* drop at drop position instead of cursor position */
 	gint link_management; 	/* perform link management */
 	gint defaulthighlight;		/* highlight documents by default */
 	gint cont_highlight_full; 	/* if you want to highlight the full text or just the line */
 	gint cont_highlight_update;	/* update the syntax highlighting continuous */
-	gchar *default_charset;
+	gchar *html_ver;
 	GList *external_filters;	/* external filter in list */
 	GList *external_commands;	/* external filter in list */
+	GList *cust_menu; 		/* entries in the custom menu */
 	GList *syntax_configstrings;	/* the strings containing the colors and patters */
+
 #ifdef WITH_SPC
 	/* spell checker options */
 	gchar *cfg_spc_cline      ;  /* spell checker command line */
@@ -97,10 +145,64 @@ typedef struct {
                                      may be part of a word ? */
 	gchar* spc_esc_chars      ;  /* which ones ? */
 	gint   spc_use_pers_dict  ;  /* use a personal dictionary */
-	gchar* spc_pers_dict      ;  /* which one ? */                           
+	gchar* spc_pers_dict      ;  /* which one ? */
+   gint   spc_use_input_encoding ;  /* use input encoding */
+   gchar* spc_input_encoding     ;  /* wich one ? */
+   gint   spc_output_html_chars  ; /* output html chars ? (like &aacute;)*/
+#endif
+	gchar *backup_filestring;  /* the string to append to the backup file */
+	gint backup_file; 			/* wheather to use a backup file */
+	gint allow_multi_instances; /* allow multiple instances of the same file */
+
+	/* key conversion */
+	gint conv_ctrl_enter;		/* convert control-enter key press */
+	gchar *ctrl_enter_text;		/* inserted text */
+	gint conv_shift_enter;		/* convert shift-enter key press */
+	gchar *shift_enter_text;	/* inserted text */
+	gint conv_special_char;		/* convert ctrl-'<','>','&' */
+#ifdef WITH_MSG_QUEUE
+	gint open_in_running_bluefish; /* open commandline documents in already running session*/
+#endif /* WITH_MSG_QUEUE */
+} Tproperties;
+
+#ifdef WITH_INTERNAL_PREVIEW
+typedef struct {
+	GtkWidget *win;
+	GtkWidget *html;
+} Tpreview;
 #endif
 
-} Tproperties;
+typedef enum {
+FILTER_ALL,
+FILTER_WEB,
+FILTER_IMAGES,
+FILTER_DOCUMENTS
+} Tfilebrowser_filter;
+
+
+typedef struct {
+	gchar *curdir;
+	GtkWidget *dir_history;
+	GtkWidget *dir_list;
+	gint dir_list_lastrow;
+	GtkWidget *file_list;
+	gint file_list_lastrow;
+	gint file_list_selectedrow;
+	GList *dir_history_menu;
+	Tfilebrowser_filter filter_type;
+	GList *directory_history;
+} Tfilebrowser;
+
+/* identifying files */
+#define TYPE_UNKNOWN 0
+#define TYPE_IMAGE 1
+#define TYPE_PHP 2
+#define TYPE_HTML 3
+#define TYPE_DIR 4
+#define TYPE_JSCRIPT 5
+#define TYPE_CSS 6
+#define TYPE_OTHER 7
+#define TYPES_COUNT 8
 
 typedef struct {
 	Tproject current_project;
@@ -111,10 +213,23 @@ typedef struct {
 	gint last_notebook_page;	/* this is the last used, this can be used to track down if the notebook page really changed */
 	GtkWidget *statusbar;
 	GtkWidget *menubar;
+	Tfilebrowser filebrowser;
+	gchar **identify_filters[TYPES_COUNT];
 	GtkWidget *main_window;
-	GList *syntax_structs;		/* the props.syntax_configstrings parsed into structs with GdkColor and regex pattern */
+	GtkWidget *cmenu;
+	GList *cmenu_arrays; /* custom menu strctures */
+#ifdef WITH_INTERNAL_PREVIEW
+	Tpreview *preview;
+#endif
+	void *syntax_structs;		/* pointer to an array of struct : the props.syntax_configstrings parsed into structs with GdkColor and regex pattern */
+	gint syntax_structs_n;		/* number of structs pointed by syntax_structs */
 	GList *command_strings;
 	GList *filter_strings;
+	GList *recent_files;
+	GdkColormap *colormap;
+#ifdef WITH_MSG_QUEUE
+	gint msgid;
+#endif /* WITH_MSG_QUEUE */
 } Tmain;
 
 extern Tmain *main_v;
@@ -122,28 +237,10 @@ extern Tmain *main_v;
 /* for exiting bluefish */
 void cya_later(GtkWidget * widget, gpointer data);
 
-/* changes the current working directory to the directory of this filename */
-gboolean change_dir(gchar * filename);
-
-/* removes the first of a char in a string, making it a \0 */
-gchar *trunc_on_char(gchar * string, gchar which_char);
-
-/* strips the path of the filename */
-char *strip_filename(gchar * string);
-
-/* strips the path that is common for both files */
-gchar *strip_common_path(char *image_fn, char *html_fn);
-gchar *most_efficient_filename(gchar *filename);
-gchar *create_relative_link_to(gchar * current_filepath, gchar * link_to_filepath);
-gchar *create_full_path(gchar * filename, gchar *basedir);
-/* returns 1 if the file is readable */
-gint file_exists_and_readable(gchar * filename);
+void load_identify_filters();
+gint identify_by_filename(gchar *filename);
 
 /* runs through all the events in the gtk and gdk queue */
 void flush_queue(void);
 
-/* this function starts a file select dialog, and returns the selected file
-   or returns NULL if non selected */
-gchar *return_file(gchar * set_file);
-GList *return_files(gchar * setfile);
 #endif							/* __BLUEFISH_H_ */
