@@ -24,9 +24,7 @@
 
 #include "config.h"
 
-#ifdef ENABLE_NLS
-#include <libintl.h>
-#endif
+#define NEW_CS3
 
 #include "intl.h"
 
