@@ -31,28 +31,52 @@ typedef struct {
 
 void document_set_fonts(gchar * textfontstring, gchar * labelfontstring, Tdocument * document);
 void document_set_wrap(gint word_wrap, gint line_wrap, Tdocument * document);
-
-void file_to_textbox(gchar * filename);
-void textbox_to_file(gchar * filename);
-void doc_insert_text_cb(GtkEditable * editable, const char *text, int len, int
-						*pos, gpointer d);
-void doc_delete_text_cb(GtkEditable * editable, int start, int end, gpointer d);
-gboolean highlight_once(Tdocument * whichdoc);
-void document_need_highlighting(Tdocument * whichdoc);
-void destroy_current_document(void);
+void document_force_def_style(Tdocument *doc, gint do_force);
+void doc_need_highlighting(Tdocument * whichdoc);
+void doc_save_selection(Tdocument *doc);
+void doc_restore_selection(Tdocument *doc);
+gint find_filename_in_documentlist(gchar *filename);
+gint position_to_linenum(gint position);
+void go_to_line(gint linenum, gint select_line);
 void copy_cb(GtkWidget * w, gpointer data);
 void paste_cb(GtkWidget * w, gpointer data);
 void cut_cb(GtkWidget * w, gpointer data);
 void sel_all_cb(GtkWidget * w, gpointer data);
+void selection_to_uppercase_cb(GtkWidget * w, gpointer data);
+void selection_to_lowercase_cb(GtkWidget * w, gpointer data);
+void all_documents_update_links(Tdocument * curdoc, gchar * oldfilename,gchar * newfilename);
 Ttext_positions get_positions(GtkWidget * textbox);
 void restore_positions(Ttext_positions positions, GtkWidget * textbox);
+
 void doc_replace_text(const gchar * newstring, gint pos, gint end, Tdocument *doc);
 void replace_text(const gchar * newstring, gint pos, gint end);
-void insert_text(const gchar * before, const gchar * after);
-Tdocument *new_document(void);
-gint position_to_linenum(gint position);
-void doc_go_to_line(gint linenum, gint select_line);
+void insert_dbl_text(const gchar * before, const gchar * after);
+void doc_insert_text(Tdocument *document, const gchar * newstring);
+
 void file_move_to_cb(GtkWidget * widget, gpointer data);
 void iso8859_1_replace_cb(GtkWidget * widget, gpointer data);
+gint doc_xy_to_cursor_pos(Tdocument *doc, gint x, gint y);
+void go_to_line_win_cb(GtkWidget * widget, gpointer data);
 void ascii_replace_cb(GtkWidget * widget, gpointer data);
+void escape_quotes_replace_cb(GtkWidget * widget, gpointer data);
+void doc_textbox_to_file(Tdocument *doc, gchar * filename);
+Tdocument *doc_new(void);
+void doc_new_with_file(gchar * filename);
+void docs_new_from_files(GList *file_list);
+void doc_destroy(Tdocument * doc);
+void doc_save(Tdocument * doc, gint do_save_as, gint do_move);
+gint doc_close(Tdocument *doc, gint warn_only);
+void file_save_cb(GtkWidget * widget, gpointer data);
+void file_save_as_cb(GtkWidget * widget, gpointer data);
+void file_move_to_cb(GtkWidget * widget, gpointer data);
+void file_open_cb(GtkWidget * widget, gpointer data);
+void file_open(gchar * filename);
+void file_insert_cb(GtkWidget * widget, gpointer data);
+void file_new_cb(GtkWidget * widget, gpointer data);
+void file_close_cb(GtkWidget * widget, gpointer data);
+gint test_only_empty_doc_left(Tdocument *doc);
+void file_close_all_cb(GtkWidget * widget, gpointer data);
+void file_save_all_cb(GtkWidget * widget, gpointer data);
+void file_print_cb(GtkWidget * widget, gpointer data);
+
 #endif
