@@ -62,22 +62,19 @@
 
 
 /********************* additions from Olivier ************************/
-static void kill_subprocess_lcb(GtkWidget * widget, gpointer data)
+/* static void kill_subprocess_lcb(GtkWidget * widget, gpointer data)
 {
-/* kill subprocess here, gpointer data is the same as the one 
-you used to call the dialog */
 	gtk_widget_destroy(GTK_WIDGET(gtk_widget_get_toplevel(widget)));
 	g_free(data);
 }
 
 static void close_error_dialog_lcb(GtkWidget * widget, gpointer data)
 {
-/* nothing happens */
 	gtk_widget_destroy(GTK_WIDGET(gtk_widget_get_toplevel(widget)));
 	g_free(data);
-}
+} */
 
-static void subprocess_error_dialog(char const *message, int enable_killing, gpointer data)
+/* static void subprocess_error_dialog(char const *message, int enable_killing, gpointer data)
 {
 	GtkWidget *win, *hbox, *vbox, *button;
 
@@ -94,7 +91,7 @@ static void subprocess_error_dialog(char const *message, int enable_killing, gpo
 		gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 	}
 	gtk_widget_show_all(win);
-}
+} */
 
 /********************* ********************** ************************/
 
