@@ -16,14 +16,31 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-
 #include "default_include.h"
 
 #include <locale.h>
 
+#include <stdlib.h> /* strtod() */
+
+#include "gtk_easy.h"
 #include "bluefish.h"
+#include "bf_lib.h"
 #include "pixmaps.h"
 
+/* void flush_queue(void)
+ * handles the gtk and gdk queue
+ */
+void flush_queue(void)
+{
+	while (gtk_events_pending()) {
+		gtk_main_iteration();
+	}
+	while (gdk_events_pending()) {
+		gdk_flush();
+	}
+}
+
+
 /*
  * Function: error_dialog
  * Arguments:
@@ -151,6 +168,14 @@ GtkWidget *boxed_combo_with_popdown(gchar * setstring, GList * which_list, gint
 	return returnwidget;
 }
 
+GtkWidget *combo_with_popdown_sized(gchar * setstring, GList * which_list, gint editable, gint width) {
+	GtkWidget *returnwidget;
+	
+	returnwidget = combo_with_popdown(setstring, which_list, editable);
+	gtk_widget_set_usize(returnwidget, width,0);
+	return returnwidget;
+}
+
 /*
  * Function: entry_with_text
  * Arguments:
@@ -224,6 +249,47 @@ GtkWidget *boxed_radiobut_with_value(gchar *labeltext, gint enabled, GtkRadioBut
 	return returnwidget;
 }
 
+static gint is_int(gfloat testval) {
+	DEBUG_MSG("is_int, (int)testval=%d\n", (int)testval);
+	if ((gfloat)(((int) testval) - ((gfloat) testval )) == 0) {
+		DEBUG_MSG("is_int, %f int!\n", testval);
+		return 1;
+	} else {
+		DEBUG_MSG("is_int, %f float!\n", testval);
+		return 0;
+	}
+}
+
+
+GtkWidget *spinbut_with_value(gchar *value, gfloat lower, gfloat upper, gfloat step_increment, gfloat page_increment) {
+	GtkAdjustment *adj;
+	GtkWidget *returnwidget;
+	guint digits;
+	double fvalue=0;
+
+	if (value) {
+		fvalue = strtod(value, NULL);
+	}
+	adj = (GtkAdjustment *) gtk_adjustment_new((gfloat)fvalue, (gfloat)lower, (gfloat)upper, step_increment, page_increment, 0.0);
+	digits = (is_int(lower) ? 0 : 2);
+	returnwidget = gtk_spin_button_new(adj, step_increment, digits);
+	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(returnwidget), FALSE);
+	gtk_spin_button_set_snap_to_ticks(GTK_SPIN_BUTTON(returnwidget), FALSE);
+	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(returnwidget), GTK_UPDATE_IF_VALID);
+	if (!value) {
+		gtk_entry_set_text(GTK_ENTRY(GTK_SPIN_BUTTON(returnwidget)), "");
+	}
+	
+	return returnwidget;
+}
+/* THIS ONE IS NEVERUSED
+GtkWidget *boxed_spinbut_with_value(gchar *value, gfloat lower, gfloat upper, gfloat step_increment, gfloat page_increment, GtkWidget *box) {
+	GtkWidget *returnwidget;
+	returnwidget = spinbut_with_value(value, lower, upper, step_increment, page_increment);
+	gtk_box_pack_start(GTK_BOX(box), returnwidget, FALSE, FALSE, 0);
+	return returnwidget;
+}
+*/
 /*
  * Function: window_with_title
  * Arguments:
@@ -252,6 +318,14 @@ GtkWidget *window_with_title(gchar * title, GtkWindowPosition position, GtkWindo
 	}
 	return returnwidget;
 }
+/* GtkWindowPosition can be 
+GTK_WIN_POS_NONE
+GTK_WIN_POS_CENTER
+GTK_WIN_POS_MOUSE */
+/* GtkWindowType can be
+GTK_WINDOW_TOPLEVEL
+GTK_WINDOW_DIALOG
+GTK_WINDOW_POPUP */
 
 /*
  * Function: window_full
@@ -259,6 +333,7 @@ GtkWidget *window_with_title(gchar * title, GtkWindowPosition position, GtkWindo
  * 	title - the title string
  * 	position - the gtk window position
  * 	type - the gtk window type
+ * 	borderwidth - border width
  * 	close_func - the callback function when closing the window
  * 	close_data - the gpointer data passed to the closefunc
  * Return value:
@@ -266,8 +341,9 @@ GtkWidget *window_with_title(gchar * title, GtkWindowPosition position, GtkWindo
  * Description:
  * 	Create new window with title, callback functions and some more settings
  */
-GtkWidget *window_full(gchar * title, GtkWindowPosition position, GtkWindowType type, gint borderwidth, GtkSignalFunc close_func,
-					   gpointer close_data)
+GtkWidget *window_full(gchar * title, GtkWindowPosition position
+			, GtkWindowType type, gint borderwidth, GtkSignalFunc close_func
+			, gpointer close_data)
 {
 
 	GtkWidget *returnwidget;
@@ -337,9 +413,21 @@ GtkWidget *apply_font_style(GtkWidget * this_widget, gchar * fontstring)
 	GdkFont *tmpfont;
 	GtkStyle *style;
 
-	DEBUG_MSG("apply_font_style, started\n");
-	if ((style = gtk_rc_get_style(main_v->main_window)) == NULL) {
-		style = gtk_style_new();
+	if (GTK_WIDGET_REALIZED(this_widget)) {
+		DEBUG_MSG("apply_font_style, started with widgets own style\n");
+		style = gtk_style_copy(gtk_widget_get_style (GTK_WIDGET(this_widget)));
+	} else {
+		DEBUG_MSG("apply_font_style, started with theme default style\n");
+		style = gtk_rc_get_style (GTK_WIDGET(this_widget));
+	}
+	if (style == NULL) {
+		DEBUG_MSG("apply_font_style, using rc_get_style from main window");
+		style = gtk_rc_get_style (GTK_WIDGET(main_v->main_window));
+	}
+	if (style == NULL) {
+		style = gtk_style_new ();
+	} else {
+		style = gtk_style_copy (style);
 	}
 	tmpfont = style->font;
 
@@ -380,6 +468,7 @@ GtkWidget *bf_stock_button(const gchar * Text, GtkSignalFunc func, gpointer func
 
 	button = gtk_button_new_with_label(Text);
 	g_return_val_if_fail(button, NULL);
+	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
 	gtk_signal_connect(GTK_OBJECT(button), "clicked", func, func_data);
 	DEBUG_MSG("bf_stock_button, func_data=%p\n", func_data);
 	return button;
@@ -437,7 +526,6 @@ static void file_but_clicked_lcb(GtkWidget * widget, GtkWidget * which_entry)
 	tmpstring = return_file(NULL);
 	if (tmpstring != NULL) {
 		if (main_v->current_document->filename != NULL) {
-/*			tmp2string = strip_common_path(tmpstring, main_v->current_document->filename); */
 			tmp2string = create_relative_link_to(main_v->current_document->filename, tmpstring);
 			g_free(tmpstring);
 			tmpstring = tmp2string;
@@ -477,66 +565,196 @@ GtkWidget *file_but_new(GtkWidget * which_entry, GtkWidget * win, gint full_path
 	return file_but;
 }
 
-/************************************************************************/
+
+/**************************************************************************/
+
+#define MAX_MUBUDI_BUT 9
+typedef struct {
+	GtkWidget *win;
+	GtkWidget *buts[MAX_MUBUDI_BUT];
+	gint retval;
+} Tmubudi;
+
+static void mubudi_destroy_lcb(GtkWidget * widget, GdkEvent *event, 	Tmubudi *mbd) {
+	DEBUG_MSG("mubudi_destroy_lcb, quitting gtk main loop\n");
+	gtk_main_quit();
+}
+
+static void mubudi_but_clicked_lcb(GtkWidget * widget, Tmubudi *mbd)
+{
+	gint i=0;
+
+	while (mbd->buts[i]) {
+		if (mbd->buts[i] == widget) {
+			mbd->retval = i;
+			DEBUG_MSG("mubudi_but_clicked_lcb, retval=%d\n", i);
+		}
+		DEBUG_MSG("mubudi_but_clicked_lcb, checking %d, if %p=%p\n", i, mbd->buts[i], widget);
+		i++;
+	}
+	mubudi_destroy_lcb(NULL, NULL, mbd);
+}
 
 /*
- * Function: close_save_cancel_dialog
+ * Function: multi_button_dialog
  * Arguments:
  * 	title - title string
+ * 	defval = default value (when window is closed)
  * 	label - label string
+ * 	button - NULL terminated array of strings, containing the button labels
  * Return value:
- * 	1 - close, 2 - save, 3 - cancel
+ * 	the index num of the button label which was clicked
+ * 	so 0 for the first label, 1 for the 2nd label, etc.
  * Description:
- * 	create dialog which returns a close, save or cancel
+ * 	create dialog with several buttons which returns a value
  */
-static void close_clicked_lcb(GtkWidget * widget, gpointer data)
-{
-	*(gint *) data = 1;
-	gtk_main_quit();
-}
 
-static void save_clicked_lcb(GtkWidget * widget, gpointer data)
-{
-	*(gint *) data = 2;
-	gtk_main_quit();
-}
+gint multi_button_dialog(gchar *title, gint defval, gchar *label, gchar **buttons) {
 
-static void cancel_clicked_lcb(GtkWidget * widget, gpointer data)
-{
-	*(gint *) data = 3;
-	gtk_main_quit();
-}
+	Tmubudi *mbd;
 
-gint close_save_cancel_dialog(gchar * title, gchar * label)
-{
-
-	GtkWidget *win, *but1, *but2, *but3;
 	GtkWidget *vbox, *hbox;
-	gint retval = 0;
+	gint retval, count;
+
+	mbd = g_malloc0(sizeof(Tmubudi));
+	mbd->retval = defval;
+
+	mbd->win = window_full(title, GTK_WIN_POS_CENTER, GTK_WINDOW_DIALOG, 5,
+			mubudi_destroy_lcb, mbd);
+	gtk_grab_add(mbd->win);
 
-	win = window_with_title(title, GTK_WIN_POS_CENTER, GTK_WINDOW_DIALOG, 5);
-	gtk_grab_add(win);
-	gtk_signal_connect(GTK_OBJECT(win), "delete_event", GTK_SIGNAL_FUNC(cancel_clicked_lcb), &retval);
 	vbox = gtk_vbox_new(FALSE, 0);
-	gtk_container_add(GTK_CONTAINER(win), vbox);
+	gtk_container_add(GTK_CONTAINER(mbd->win), vbox);
 	gtk_box_pack_start(GTK_BOX(vbox), gtk_label_new(label), FALSE, FALSE, 0);
 	hbox = gtk_hbox_new(TRUE, 5);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-	but1 = bf_stock_button("Close", close_clicked_lcb, (gpointer) & retval);
-	but2 = bf_stock_button("Save", save_clicked_lcb, (gpointer) & retval);
-	but3 = bf_stock_button("Cancel", cancel_clicked_lcb, (gpointer) & retval);
-	GTK_WIDGET_SET_FLAGS(but1, GTK_CAN_DEFAULT);
-	GTK_WIDGET_SET_FLAGS(but2, GTK_CAN_DEFAULT);
-	GTK_WIDGET_SET_FLAGS(but3, GTK_CAN_DEFAULT);
-	gtk_box_pack_start(GTK_BOX(hbox), but1, TRUE, TRUE, 5);
-	gtk_box_pack_start(GTK_BOX(hbox), but2, TRUE, TRUE, 5);
-	gtk_box_pack_start(GTK_BOX(hbox), but3, TRUE, TRUE, 5);
-	gtk_widget_show_all(win);
+	count = 0;
+	while (buttons[count]) {
+		mbd->buts[count] = bf_stock_button(_(buttons[count]), mubudi_but_clicked_lcb, mbd);
+		if (count == defval) {
+			gtk_window_set_default(GTK_WINDOW(mbd->win), mbd->buts[count]);
+		}
+		gtk_box_pack_start(GTK_BOX(hbox), mbd->buts[count], TRUE, TRUE, 5);
+		count++;
+	}
+
+	gtk_widget_show_all(mbd->win);
+	gtk_window_set_transient_for(GTK_WINDOW(mbd->win), GTK_WINDOW(main_v->main_window));
 	gtk_main();
-	gtk_signal_handlers_destroy(GTK_OBJECT(win));
-	gtk_widget_destroy(win);
+	window_destroy(mbd->win);
+	retval = mbd->retval;
+	free(mbd);
 	return retval;
+
+}
+/************************************************************************/
+/************    FILE SELECTION FUNCTIONS  ******************************/
+/************************************************************************/
+typedef struct {
+	gint multipleselect;
+	gchar *filename_to_return;
+	GList *filenames_to_return;
+	GtkWidget *fs;
+} Tfileselect;
+
+static void fs_ok_clicked_lcb(GtkWidget * widget, Tfileselect *fileselect)
+{
+	gchar *dirname, *selected_file;
+
+	selected_file = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselect->fs));
+
+	if (file_is_dir(selected_file)) {
+		DEBUG_MSG("fs_ok_clicked_lcb,file_is_dir said %s is a dir!!!!\n", selected_file);
+		dirname = ending_slash(selected_file);
+		gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselect->fs), dirname);
+		gtk_entry_set_text(GTK_ENTRY(GTK_FILE_SELECTION(fileselect->fs)->selection_entry), "");
+		g_free(dirname);
+		return;
+	} else {
+		DEBUG_MSG("fs_ok_clicked_lcb,file_is_dir said %s is NOT a dir.\n", selected_file);
+	}
+
+	if (fileselect->multipleselect) {
+		/* multiple files allowed --> scan trough the list for selections */
+		dirname = g_dirname(selected_file);
+		{
+		gint rownum = 0;
+		gchar *temp, *totalfilename;
+		GList *row = GTK_CLIST(GTK_FILE_SELECTION(fileselect->fs)->file_list)->row_list;
+		while (row) {
+			if (GTK_CLIST_ROW(row)->state == GTK_STATE_SELECTED) {
+				gtk_clist_get_text(GTK_CLIST(GTK_FILE_SELECTION(fileselect->fs)->file_list), rownum, 0, &temp);
+				DEBUG_MSG("fs_ok_clicked_lcb, temp=%s\n", temp);
+				totalfilename = g_strconcat(dirname, "/", temp, NULL);
+				fileselect->filenames_to_return = g_list_append(fileselect->filenames_to_return, totalfilename);
+			}
+			DEBUG_MSG("fs_ok_clicked_lcb, rownum=%d\n", rownum);
+			rownum++;
+			row = g_list_next(row);
+		}
+		}
+		g_free(dirname);
+	} else {
+		/* NO multiple files allowed --> return just one file */
+		fileselect->filename_to_return = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselect->fs)));
+	}
+	gtk_main_quit();
+	window_destroy(GTK_WIDGET(fileselect->fs));
 }
 
+static void close_modal_window_lcb(GtkWidget * widget, gpointer window)
+{
+	gtk_main_quit();
+	window_destroy(window);
+}
+
+static void fileselectwin(gchar *setfile, Tfileselect *fileselect, gchar *title) {
+
+	DEBUG_MSG("fileselectwin, started\n");
+	fileselect->fs = gtk_file_selection_new(title);
+	gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(fileselect->fs));
+	gtk_signal_connect(GTK_OBJECT(fileselect->fs), "destroy", close_modal_window_lcb, fileselect->fs);
+	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fileselect->fs)->cancel_button),
+					   "clicked", close_modal_window_lcb, fileselect->fs);
+	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fileselect->fs)->ok_button), "clicked", (GtkSignalFunc) fs_ok_clicked_lcb, fileselect);
+	if (fileselect->multipleselect) {
+		gtk_clist_set_selection_mode(GTK_CLIST(GTK_FILE_SELECTION(fileselect->fs)->file_list), GTK_SELECTION_EXTENDED);
+	}
+	if (setfile) {
+		gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselect->fs), setfile);
+	}
+	gtk_window_set_wmclass(GTK_WINDOW(fileselect->fs), "Bluefish", "fileselect");
+	gtk_widget_show(fileselect->fs);
+	gtk_grab_add(GTK_WIDGET(fileselect->fs));
+	gtk_widget_realize(GTK_WIDGET(fileselect->fs));
+	gtk_window_set_transient_for(GTK_WINDOW(fileselect->fs), GTK_WINDOW(main_v->main_window));
+}
+
+gchar *return_file_w_title(gchar * setfile, gchar *title) {
+	Tfileselect fileselect={0, NULL, NULL, NULL};
+
+	fileselectwin(setfile, &fileselect, title);
+	gtk_main();
+	return fileselect.filename_to_return;
+}
+
+gchar *return_file(gchar * setfile) {
+	return return_file_w_title(setfile, _("Select file"));
+}
+
+GList *return_files_w_title(gchar * setfile, gchar *title) {
+	Tfileselect fileselect={1, NULL, NULL, NULL};
+	
+	fileselectwin(setfile, &fileselect, title);
+	gtk_main();
+	return fileselect.filenames_to_return;
+}
+
+GList *return_files(gchar * setfile) {
+	return return_files_w_title(setfile, _("Select files"));
+}
+
+
+
 /************************************************************************/
