@@ -36,12 +36,14 @@ void setup_toggle_item(GtkItemFactory * ifactory, gchar * path, gint state);
 void string_apply(gchar ** config_var, GtkWidget * entry);
 GtkWidget *combo_with_popdown(gchar * setstring, GList * which_list, gint editable);
 GtkWidget *boxed_combo_with_popdown(gchar * setstring, GList * which_list, gint editable, GtkWidget *box);
+GtkWidget *combo_with_popdown_sized(gchar * setstring, GList * which_list, gint editable, gint width);
 GtkWidget *entry_with_text(gchar * setstring, gint max_lenght);
 GtkWidget *boxed_entry_with_text(gchar * setstring, gint max_lenght, GtkWidget *box);
 GtkWidget *checkbut_with_value(gchar *labeltext, gint which_config_int);
 GtkWidget *boxed_checkbut_with_value(gchar *labeltext, gint which_config_int, GtkWidget * box);
 GtkWidget *radiobut_with_value(gchar *labeltext, gint enabled, GtkRadioButton *prevbut);
 GtkWidget *boxed_radiobut_with_value(gchar *labeltext, gint enabled, GtkRadioButton *prevbut, GtkWidget *box);
+GtkWidget *spinbut_with_value(gchar *value, gfloat lower, gfloat upper, gfloat step_increment, gfloat page_increment);
 GtkWidget *window_with_title(gchar * title, GtkWindowPosition position, GtkWindowType type, gint borderwidth);
 GtkWidget *window_full(gchar * title, GtkWindowPosition position, GtkWindowType type, gint borderwidth, GtkSignalFunc close_func,
 					   gpointer close_data);
@@ -53,5 +55,11 @@ GtkWidget *bf_stock_button(const gchar * Text, GtkSignalFunc func, gpointer func
 GtkWidget *bf_stock_ok_button(GtkSignalFunc func, gpointer func_data);
 GtkWidget *bf_stock_cancel_button(GtkSignalFunc func, gpointer func_data);
 GtkWidget *file_but_new(GtkWidget * which_entry, GtkWidget * win, gint full_pathname);
-gint close_save_cancel_dialog(gchar * title, gchar * label);
+gint multi_button_dialog(gchar *title, gint defval, gchar *label, gchar **buttons);
+
+gchar *return_file_w_title(gchar * setfile, gchar *title);
+gchar *return_file(gchar * setfile);
+GList *return_files_w_title(gchar * setfile, gchar *title);
+GList *return_files(gchar * setfile);
+
 #endif
