@@ -52,30 +52,44 @@
 
 /***********************************************************************/
 
+typedef gshort Thp_index;
+
 typedef struct {
 	GdkColor color;
 	regex_t reg_comp_pat;
 } Thighlight_pattern;
 
+/* for access to main_v->main_v->syntax_structs */
+#define HIPA_ARRAY_PTR(p) ((Thighlight_pattern*)(p))
+
 /***********************************************************************/
 
+static gchar **create_array(gchar *color, gchar *pattern) {
+	gchar **array;
+
+	array = g_malloc(3* sizeof(gchar *));
+	array[0] = g_strdup(color);
+	array[1] = g_strdup(pattern);
+	array[2] = NULL;
+	return array;
+}
+
 void reset_syntax_highlighting_lcb(GtkWidget * widget, gpointer data)
 {
-	free_stringlist(main_v->props.syntax_configstrings);
+	free_arraylist(main_v->props.syntax_configstrings);
+	
 	main_v->props.syntax_configstrings = NULL;
-
-	main_v->props.syntax_configstrings = add_to_stringlist(main_v->props.syntax_configstrings, "#000080 <[^(!|%|?)][^>]*>");
-	main_v->props.syntax_configstrings = add_to_stringlist(main_v->props.syntax_configstrings, "#ee0000 (<\\\?)[^?]*(\\\?>)");
-	main_v->props.syntax_configstrings = add_to_stringlist(main_v->props.syntax_configstrings, "#ee0000 (<\\\?)[^>]*(\\\?>)");
-	main_v->props.syntax_configstrings = add_to_stringlist(main_v->props.syntax_configstrings, "#bd0000 (<[/]*script[^>]*>)");
-	main_v->props.syntax_configstrings =
-		add_to_stringlist(main_v->props.syntax_configstrings, "#00AAAA (<[/]*table[^>]*>)|(<[/]*td[^>]*>)|(<[/]*tr[^>]*>)|(<[/]*th[^>]*>)");
-	main_v->props.syntax_configstrings =
-		add_to_stringlist(main_v->props.syntax_configstrings,
-						  "#4080bf (<[/]*form[^>]*>)|(<input[^>]*>)|(<[/]*select[^>]*>)|(<[/]*textarea[^>]*>)|(<[/]*option[^>]*>)");
-	main_v->props.syntax_configstrings = add_to_stringlist(main_v->props.syntax_configstrings, "#eead0e (<[/]*a[^>]*>)|(<[/]*img[^>]*>)");
-	main_v->props.syntax_configstrings = add_to_stringlist(main_v->props.syntax_configstrings, "#969696 (<!--)[^>]*-->");
-	main_v->props.syntax_configstrings = add_to_stringlist(main_v->props.syntax_configstrings, "#22AA00 \"[^\"]*\"");
+	main_v->props.syntax_configstrings = g_list_append(main_v->props.syntax_configstrings, create_array("#000080", "<[^(!|%|?)][^>]*>"));
+	main_v->props.syntax_configstrings = g_list_append(main_v->props.syntax_configstrings, create_array("#ee0000", "(<\\\?)[^?]*(\\\?>)"));
+	main_v->props.syntax_configstrings = g_list_append(main_v->props.syntax_configstrings, create_array("#ee0000", "(<\\\?)[^>]*(\\\?>)"));
+	main_v->props.syntax_configstrings = g_list_append(main_v->props.syntax_configstrings, create_array("#bd0000", "(<[/]*script[^>]*>)"));
+	main_v->props.syntax_configstrings = g_list_append(main_v->props.syntax_configstrings, create_array("#00AAAA", "(<[/]*table[^>]*>)|(<[/]*td[^>]*>)|(<[/]*tr[^>]*>)|(<[/]*th[^>]*>)"));
+	main_v->props.syntax_configstrings = g_list_append(main_v->props.syntax_configstrings, create_array("#4080bf", "(<[/]*form[^>]*>)|(<input[^>]*>)|(<[/]*select[^>]*>)|(<[/]*textarea[^>]*>)|(<[/]*option[^>]*>)"));
+	main_v->props.syntax_configstrings = g_list_append(main_v->props.syntax_configstrings, create_array("#eead0e", "(<[/]*a( )?[^>]*>)|(<[/]*img[^>]*>)"));
+	main_v->props.syntax_configstrings = g_list_append(main_v->props.syntax_configstrings, create_array("#969696", "(<!--)[^>]*-->"));
+	main_v->props.syntax_configstrings = g_list_append(main_v->props.syntax_configstrings, create_array("#1D9200", "(&)[a-zA-Z]+(;)"));
+	main_v->props.syntax_configstrings = g_list_append(main_v->props.syntax_configstrings, create_array("#22AA00", "\"[^\"]*\""));
+	make_syntax_struct_list();
 }
 
 
@@ -105,15 +119,13 @@ static regmatch_t checkreg(char *htmlstring, regex_t * pregp)
  */
 #define CRAPPY_FIX
 
-static void hlregex(GtkWidget * textbox, Thighlight_pattern * hipa, gchar * fulltext)
+static void hlregex(Thighlight_pattern * hipa, Thp_index hipa_index, gchar * fulltext, Thp_index *fulltext_attr, gint fulltext_len)
 {
 	regmatch_t tagmatch;
 	gshort loop = 1;
-	gchar *tmpbuf;
 	gint pos = 0;
-	gint maxlen;
+	gint i;
 
-	maxlen = strlen(fulltext);
 	while (loop == 1) {
 		tagmatch = checkreg(&fulltext[pos], &hipa->reg_comp_pat);
 
@@ -126,110 +138,111 @@ static void hlregex(GtkWidget * textbox, Thighlight_pattern * hipa, gchar * full
 				("position in text is=%d, match start=%d, match end=%d, match length=%d\n",
 				 pos, pos + tagmatch.rm_so, pos + tagmatch.rm_eo, tagmatch.rm_eo - tagmatch.rm_so);
 #endif
-#ifdef CRAPPY_FIX_OLD
-#if GTK_GT_7 == 0
-			gtk_text_thaw(GTK_TEXT(textbox));
-#endif
-#endif
-			gtk_text_set_point(GTK_TEXT(textbox), pos + tagmatch.rm_so);
-#ifdef CRAPPY_FIX_OLD
-#if GTK_GT_7 == 0
-			gtk_text_freeze(GTK_TEXT(textbox));
-#endif
-#endif
-			gtk_text_forward_delete(GTK_TEXT(textbox)
-									, tagmatch.rm_eo - tagmatch.rm_so);
-			tmpbuf = g_strndup(&fulltext[pos + tagmatch.rm_so]
-							   , tagmatch.rm_eo - tagmatch.rm_so);
-			gtk_text_insert(GTK_TEXT(textbox), NULL, &hipa->color, NULL, tmpbuf, -1);
-			g_free(tmpbuf);
-			pos = pos + tagmatch.rm_eo;
-			if (pos >= maxlen) {
-				loop = 0;
+			for (i = tagmatch.rm_so; i < tagmatch.rm_eo; i++) {
+				if (pos + i >= fulltext_len) {
+					loop = 0;
+					break;
+				} else {
+					fulltext_attr[pos + i] = hipa_index + 1;
+				}
 			}
+			pos = pos + tagmatch.rm_eo;
 		}
 
 	}
 }
 
-static void hlregex_line(GtkWidget * textbox, Thighlight_pattern * hipa, gint startol, gint endol, gchar * line)
+static void insert_highlighted_text(GtkWidget * textbox, Thighlight_pattern *hipalist, gchar * fulltext, Thp_index *fulltext_attr, gint fulltext_len)
 {
-	regmatch_t tagmatch;
-	gshort loop = 1;
+	gint start_pos = 0, end_pos = 0;
+	Thp_index hipa_index = fulltext_attr[0];
 	gchar *tmpbuf;
-	guint smallpos = 0;
-	gint maxlen, buflen;
+	Thighlight_pattern *hipa;
+	GdkColor *hipa_color;
 
-	maxlen = endol - startol;
-
-	while (loop == 1) {
-		tagmatch = checkreg(&line[smallpos], &hipa->reg_comp_pat);
-
-		if (tagmatch.rm_so == -1) {
-			loop = 0;
-		} else {
-			buflen = tagmatch.rm_eo - tagmatch.rm_so;
-			gtk_text_set_point(GTK_TEXT(textbox), startol + smallpos + tagmatch.rm_so);
-			gtk_text_forward_delete(GTK_TEXT(textbox) ,buflen);
-			tmpbuf = g_strndup(&line[smallpos + tagmatch.rm_so]
-							   ,buflen);
-			gtk_text_insert(GTK_TEXT(textbox), NULL, &hipa->color, NULL, tmpbuf, buflen);
-
-#ifdef CRAPPY_FIX
-#if GTK_GT_7 == 0
-			gtk_text_thaw(GTK_TEXT(textbox));
-			gtk_text_freeze(GTK_TEXT(textbox));
-#endif
-#endif
-
-			g_free(tmpbuf);
-			smallpos = smallpos + tagmatch.rm_eo;
-			if (smallpos >= maxlen) {
-				loop = 0;
+	for (end_pos = 0; end_pos < fulltext_len; end_pos++) {
+		if (fulltext_attr[end_pos] != hipa_index) {
+			tmpbuf = g_strndup(&fulltext[start_pos], end_pos - start_pos);
+			if (hipa_index > 0) {
+				hipa = &hipalist[hipa_index-1];
+				hipa_color = &hipa->color;
+			} else {
+				hipa_color = &textbox->style->black;
 			}
+			gtk_text_insert(GTK_TEXT(textbox), NULL, hipa_color, NULL, tmpbuf, -1);
+			g_free(tmpbuf);
+			start_pos = end_pos;
+			hipa_index = fulltext_attr[end_pos];
 		}
 	}
+	if (start_pos != end_pos) {
+		tmpbuf = g_strndup(&fulltext[start_pos], end_pos - start_pos);
+		if (hipa_index > 0) {
+			hipa = &hipalist[hipa_index-1];
+			hipa_color = &hipa->color;
+		} else {
+			hipa_color = &textbox->style->black;
+		}
+		gtk_text_insert(GTK_TEXT(textbox), NULL, hipa_color, NULL, tmpbuf, -1);
+		g_free(tmpbuf);
+	}
 }
 
 
 static void start_highlighting(GtkWidget * textbox)
 {
-	GList *tmplist;
+	Thighlight_pattern *hipa;
 	gchar *fulltext;
-
+	Thp_index *fulltext_attr;
+	gint fulltext_bufflen;
+	gint textlen;
+	gint i;
 
 	DEBUG_MSG("start_highlighting, started\n");
-	fulltext = gtk_editable_get_chars(GTK_EDITABLE(textbox), 0, -1);
-
 	gtk_text_freeze(GTK_TEXT(textbox));
-
-	tmplist = g_list_first(main_v->syntax_structs);
-	while (tmplist) {
-		hlregex(textbox, (Thighlight_pattern *) tmplist->data, fulltext);
-#ifdef CRAPPY_FIX
-#if GTK_GT_7 == 0
+	fulltext = gtk_editable_get_chars(GTK_EDITABLE(textbox), 0, -1);
+	if (!fulltext) {
 		gtk_text_thaw(GTK_TEXT(textbox));
-		gtk_text_freeze(GTK_TEXT(textbox));
-#endif
-#endif
-		tmplist = g_list_next(tmplist);
+		return;
+	}
+	fulltext_bufflen = strlen(fulltext);
+	fulltext_attr = g_new0(Thp_index, fulltext_bufflen);
+	if (!fulltext_attr) {
+		g_free(fulltext);
+		gtk_text_thaw(GTK_TEXT(textbox));
+		return;
 	}
-	g_free(fulltext);
 
+	hipa = HIPA_ARRAY_PTR(main_v->syntax_structs);
+	for (i = 0; i < main_v->syntax_structs_n; i++) {
+		hlregex(hipa, i, fulltext, fulltext_attr, fulltext_bufflen);
+		hipa++;
+		DEBUG_MSG("start_highlighting, i=%d\n", i);
+	}
+
+	textlen = gtk_text_get_length(GTK_TEXT(textbox));
+	gtk_text_set_point(GTK_TEXT(textbox), 0);
+	gtk_text_forward_delete(GTK_TEXT(textbox), textlen);
+
+	insert_highlighted_text(textbox, HIPA_ARRAY_PTR(main_v->syntax_structs), fulltext, fulltext_attr, fulltext_bufflen);
+
+	g_free(fulltext);
+	g_free(fulltext_attr);
 	gtk_text_thaw(GTK_TEXT(textbox));
 
 }
 
 void line_highlighting(GtkWidget * textbox)
 {
-	GList *tmplist;
+	Thighlight_pattern *hipa;
 	gchar *line, curchar;
-	guint pos,linelen;
+	Thp_index *line_attr;
+	guint pos,linelen,line_bufflen;
 	gint startol, endol, max;
+	gint i;
 
 	DEBUG_MSG("line_highlighting textbox=%p\n", textbox);
 	gtk_text_freeze(GTK_TEXT(textbox));
-	
 	pos = gtk_editable_get_position(GTK_EDITABLE(textbox));
 	max = gtk_text_get_length(GTK_TEXT(textbox));
 
@@ -255,29 +268,32 @@ void line_highlighting(GtkWidget * textbox)
 	
 	line = gtk_editable_get_chars(GTK_EDITABLE(textbox), startol, endol);
 	if (!line) {
-		g_free(line);
 		gtk_text_thaw (GTK_TEXT(textbox));
 		return;
 	}
 
+	line_bufflen = strlen(line);
+	line_attr = g_new0(Thp_index, line_bufflen);
+	if (!line_attr) {
+		g_free(line);
+		gtk_text_thaw(GTK_TEXT(textbox));
+		return;
+	}
+
+	DEBUG_MSG("line_highlighting, startol=%d, endol=%d, inserting %s\n", startol, endol, line);
+
+	hipa = HIPA_ARRAY_PTR(main_v->syntax_structs);
+	for (i = 0; i < main_v->syntax_structs_n; i++) {
+		hlregex(hipa, i, line, line_attr, line_bufflen);
+		hipa++;
+	}
+
 	gtk_text_set_point(GTK_TEXT(textbox), startol);
 	gtk_text_forward_delete(GTK_TEXT(textbox), linelen);
-	DEBUG_MSG("line_highlighting, startol=%d, endol=%d, inserting %s\n", startol, endol, line);
-	gtk_text_insert(GTK_TEXT(textbox), NULL, NULL, NULL, line, linelen);
+	insert_highlighted_text(textbox, HIPA_ARRAY_PTR(main_v->syntax_structs), line, line_attr, line_bufflen);
 
-	tmplist = g_list_first(main_v->syntax_structs);
-	while (tmplist) {
-		hlregex_line(textbox, (Thighlight_pattern *) tmplist->data, startol,endol,line);
-#ifdef CRAPPY_FIX
-#if GTK_GT_7 == 0
-		gtk_text_thaw(GTK_TEXT(textbox));
-		gtk_text_freeze(GTK_TEXT(textbox));
-#endif
-#endif
-		tmplist = g_list_next(tmplist);
-	}
 	g_free(line);
-
+	g_free(line_attr);
 	gtk_text_thaw(GTK_TEXT(textbox));
 }
 
@@ -286,17 +302,21 @@ static void stop_highlighting(GtkWidget * textbox)
 {
 	gchar *buffer;
 	guint buflen;
+	guint textlen;
 	guint scrollp;
 	DEBUG_MSG("stop_highlighting, started\n");
 	gtk_text_freeze(GTK_TEXT(textbox));
 	scrollp = gtk_editable_get_position(GTK_EDITABLE(textbox));
 	buffer = gtk_editable_get_chars(GTK_EDITABLE(textbox), 0, -1);
 	buflen = strlen(buffer);
+	/* if multibyte character was used, buflen is not a length of text.
+	   so we query a length of text to the widget. */
+	textlen = gtk_text_get_length(GTK_TEXT(textbox));
 	if (buflen > 0) {
 		DEBUG_MSG("stop_highlighting, buflen=%d\n", buflen);
 		gtk_text_set_point(GTK_TEXT(textbox), 0);
 		DEBUG_MSG("stop_highlighting, point set to %d\n", gtk_text_get_point(GTK_TEXT(textbox)));
-		gtk_text_forward_delete(GTK_TEXT(textbox), buflen);
+		gtk_text_forward_delete(GTK_TEXT(textbox), textlen);
 #ifdef CRAPPY_FIX
 #if GTK_GT_7 == 0
 		gtk_text_thaw(GTK_TEXT(textbox));
@@ -407,44 +427,67 @@ void toggle_highlight_syntax_cb(GtkWidget * w, gpointer data)
 
 void make_syntax_struct_list(void)
 {
-
 	GList *tmplist;
+	gint tmplist_len;
 	gchar **splittedstring;
 	gint ret;
-	GdkColormap *colormap;
 	Thighlight_pattern *hipa;
+	gint hipa_count;
+	gint i;
 
-	DEBUG_MSG("apply_highlighting_config, started\n");
-	colormap = gdk_colormap_get_system();
+	DEBUG_MSG("make_syntax_struct_list, started\n");
 
-	tmplist = g_list_first(main_v->syntax_structs);
-	while (tmplist) {
-		hipa = (Thighlight_pattern *) tmplist->data;
-		regfree(&hipa->reg_comp_pat);
-		gdk_colormap_free_colors(colormap, &hipa->color, 1);
-		g_free(hipa);
-		tmplist = g_list_next(tmplist);
+	hipa = HIPA_ARRAY_PTR(main_v->syntax_structs);
+	if (hipa) {
+		for (i = 0; i < main_v->syntax_structs_n; i++) {
+			regfree(&hipa->reg_comp_pat);
+			gdk_colormap_free_colors(main_v->colormap, &hipa->color, 1);
+			hipa++;
+		}
 	}
-	g_list_free(main_v->syntax_structs);
+	g_free(main_v->syntax_structs);
 	main_v->syntax_structs = NULL;
+	main_v->syntax_structs_n = 0;
 
 	tmplist = g_list_first(main_v->props.syntax_configstrings);
+	if (!tmplist)
+		return;
+
+	tmplist_len = g_list_length(tmplist);
+	main_v->syntax_structs = g_new(Thighlight_pattern, tmplist_len);
+	if (!main_v->syntax_structs)
+		return;
+
+	hipa = HIPA_ARRAY_PTR(main_v->syntax_structs);
+	hipa_count = 0;
 	while (tmplist) {
-		splittedstring = g_strsplit((gchar *) tmplist->data, " ", 2);
-		DEBUG_MSG("apply_highlighting_config, splittedstring[0]=%s\n", splittedstring[0]);
-		DEBUG_MSG("apply_highlighting_config, splittedstring[1]=%s\n", splittedstring[1]);
-		hipa = g_malloc0(sizeof(Thighlight_pattern));
+		splittedstring = (gchar **) tmplist->data;
+		if (!splittedstring[0] || !splittedstring[1]) {
+			DEBUG_MSG("make_syntax_struct_list, problem with config file\n");
+			tmplist = g_list_next(tmplist);
+			continue;
+		}
+		DEBUG_MSG("make_syntax_struct_list, splittedstring[0]=%s\n", splittedstring[0]);
+		DEBUG_MSG("make_syntax_struct_list, splittedstring[1]=%s\n", splittedstring[1]);
 		ret = gdk_color_parse(splittedstring[0], &hipa->color);
 		if (ret == 1) {
 			regcomp(&hipa->reg_comp_pat, splittedstring[1], REG_EXTENDED | REG_ICASE);
-			gdk_colormap_alloc_color(colormap, &hipa->color, TRUE, TRUE);
-			main_v->syntax_structs = g_list_append(main_v->syntax_structs, hipa);
+			gdk_colormap_alloc_color(main_v->colormap, &hipa->color, FALSE, TRUE);
+			hipa++;
+			hipa_count++;
 		} else {
-			DEBUG_MSG("apply_highlighting_config, ret=%d !!!\n", ret);
-			g_free(hipa);
+			DEBUG_MSG("make_syntax_struct_list, ret=%d !!!\n", ret);
 		}
-		g_strfreev(splittedstring);
 		tmplist = g_list_next(tmplist);
 	}
-	DEBUG_MSG("apply_highlighting_config, ended\n");
+
+	if (hipa_count == 0) {
+		g_free(main_v->syntax_structs);
+		main_v->syntax_structs = NULL;
+		main_v->syntax_structs_n = 0;
+	} else {
+		main_v->syntax_structs_n = hipa_count;
+	}
+
+	DEBUG_MSG("apply_highlighting_config, ended, hipa_count = %d\n", hipa_count);
 }
