@@ -21,10 +21,12 @@
 
 #ifndef __HIGHLIGHT_H_
 #define __HIGHLIGHT_H_
-
+void reset_syntax_highlighting_lcb(GtkWidget * widget, gpointer data);
 void stop_hi_cb(GtkWidget * widget, gpointer data);
 void start_hi_cb(GtkWidget * widget, gpointer data);
 void refresh_hi_cb(GtkWidget * widget, gpointer data);
 void toggle_highlight_syntax_cb(GtkWidget * w, gpointer data);
 void make_syntax_struct_list(void);
+void line_highlighting(GtkWidget * textbox);
+void line_hi_cb(GtkWidget * widget, gpointer data);
 #endif
