@@ -31,7 +31,7 @@
 #include "bluefish.h"
 #include "init.h"
 #include "gtk_easy.h"
-#include "html.h"
+#include "html_diag.h" /* parse_html_for_dialogvalues() */
 #include "document.h"
 #include "rpopup.h"
 #include "images.h"
@@ -54,7 +54,9 @@ typedef struct {
 	GtkWidget *align;
 	GtkWidget *frame;
 	GtkWidget *custom;
-	Treplacerange *replace_range;
+	GtkWidget *usemap;
+	GtkWidget *name;
+	Treplacerange range;
 	gint tnH;
 	gint tnW;
 	gint h;
@@ -153,23 +155,45 @@ static void image_dialog(GtkWidget * widget, gpointer data, gboolean thumb)
 {
 
 	Timagewin *image_win = NULL;
-	GtkWidget *table, *scale, *hbox;
+	GtkWidget *table, *scale, *hbox, *but;
 	GtkObject *adj;
-
-	static gchar *tagitems[] = { "width", "height", "alt", "border", "src", "hspace", "vspace", "align", NULL };
-	gchar *tagvalues[9];
+	gint count=0;
+	static gchar *tagitems[] = { "width", "height", "alt", "border", "src", "hspace", "vspace", "align", "name", "usemap", NULL };
+	gchar *tagvalues[11];
 	gchar *custom = NULL;
 
 	DEBUG_MSG("image_dialog, started\n");
+
+
+	/* switch colormap to Imlib colormap, so image preview works */
+	{
+		GdkVisual	*gdkvisual;
+		GdkColormap	*gdkcmap;
+
+		gdkvisual = gdk_imlib_get_visual ();
+		gdkcmap   = gdk_imlib_get_colormap ();
+
+		if (gdkvisual && gdkcmap) {
+			gtk_widget_push_visual (gdkvisual);
+			gtk_widget_push_colormap (gdkcmap);
+		}
+	}
+
 	image_win = g_malloc0(sizeof(Timagewin));
-	image_win->replace_range = NULL;
+
 
 	/* this is for the right click menu */
+	while (tagitems[count]) {
+		tagvalues[count] = NULL;
+		count++;
+	}
 	if ((data != NULL) && (widget == NULL)) {
-		image_win->replace_range = values_for_dialog(tagitems, tagvalues, &custom, (Ttagpopup *) data);
-		DEBUG_MSG("image_dialog, image_win->replace_range=%p\n", image_win->replace_range);
+		parse_html_for_dialogvalues(tagitems, tagvalues, &custom, ((Ttagpopup *)data));
+		image_win->range.pos = ((Ttagpopup *)data)->pos;
+		image_win->range.len = ((Ttagpopup *)data)->len;
 	} else {
-		init_values_for_dialog(tagitems, tagvalues);
+		image_win->range.pos = -1;
+		image_win->range.len = -1;	
 	}
 	/* end for the right click menu */
 
@@ -187,16 +211,11 @@ static void image_dialog(GtkWidget * widget, gpointer data, gboolean thumb)
 /*	gtk_widget_set_usize(image_win->frame, 200, 0); */
 	gtk_box_pack_start(GTK_BOX(hbox), image_win->frame, TRUE, TRUE, 5);
 
-	table = gtk_table_new(13, 4, 1);
+	table = gtk_table_new(14, 10, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), table, TRUE, TRUE, 5);
 
 	gtk_widget_realize(image_win->pixwin);
-	image_win->location = gtk_entry_new();
-	if (tagvalues[4]) {
-		gtk_entry_set_text(GTK_ENTRY(image_win->location), tagvalues[4]);
-	} else {
-		DEBUG_MSG("image_dialog, tagvalues[4]=NULL\n");
-	}
+	image_win->location = entry_with_text(tagvalues[4], 0);
 	gtk_table_attach_defaults(GTK_TABLE(table), image_win->location, 0, 3, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(_("Image location")), 0, 2, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(table), file_but_new(image_win->location, image_win->pixwin, 0), 3, 4, 1, 2);
@@ -219,14 +238,9 @@ static void image_dialog(GtkWidget * widget, gpointer data, gboolean thumb)
 	gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(_("Height")), 2, 4, 2, 3);
 	parse_integer_for_dialog(tagvalues[1], image_win->height, NULL, image_win->height_perc);
 
-	image_win->alt = gtk_entry_new();
+	image_win->alt = entry_with_text(tagvalues[2], 0);
 	gtk_table_attach_defaults(GTK_TABLE(table), image_win->alt, 0, 4, 5, 6);
 	gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(_("Alternate text")), 0, 4, 4, 5);
-	if (tagvalues[2]) {
-		gtk_entry_set_text(GTK_ENTRY(image_win->alt), tagvalues[2]);
-	} else {
-		DEBUG_MSG("image_dialog, tagvalues[2]=NULL\n");
-	}
 
 	adj = gtk_adjustment_new((gfloat) 0, 0.0, 600.0, 1.0, 5.0, 0.0);
 	image_win->border = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 0);
@@ -251,14 +265,17 @@ static void image_dialog(GtkWidget * widget, gpointer data, gboolean thumb)
 	gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(_("Vspace")), 2, 3, 7, 8);
 	parse_integer_for_dialog(tagvalues[6], image_win->vspace, NULL, NULL);
 
-	image_win->custom = gtk_entry_new();
-	gtk_table_attach_defaults(GTK_TABLE(table), image_win->custom, 0, 4, 9, 10);
-	gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(_("Custom")), 0, 4, 8, 9);
-	if (custom) {
-		gtk_entry_set_text(GTK_ENTRY(image_win->custom), custom);
-	} else {
-		DEBUG_MSG("image_dialog, custom=NULL\n");
-	}
+	image_win->name = entry_with_text(tagvalues[8], 0);
+	gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(_("Name")), 0, 2, 8, 9);
+	gtk_table_attach_defaults(GTK_TABLE(table), image_win->name, 2, 4, 8, 9);
+
+	image_win->usemap = entry_with_text(tagvalues[9], 0);
+	gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(_("Usemap")), 0, 2, 9, 10);
+	gtk_table_attach_defaults(GTK_TABLE(table), image_win->usemap, 2, 4, 9, 10);
+
+	image_win->custom = entry_with_text(custom, 0);
+	gtk_table_attach_defaults(GTK_TABLE(table), image_win->custom, 0, 4, 11, 12);
+	gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(_("Custom")), 0, 4, 10, 11);
 
 	if (image_win->thumbnail) {
 		image_win->adjustment = GTK_ADJUSTMENT(gtk_adjustment_new(0.5, 0.001, 1.1, 0.001, 0.1, 0.1));
@@ -266,21 +283,29 @@ static void image_dialog(GtkWidget * widget, gpointer data, gboolean thumb)
 		gtk_signal_connect(GTK_OBJECT(image_win->adjustment), "value_changed", GTK_SIGNAL_FUNC(image_adjust_changed), image_win);
 		gtk_scale_set_digits(GTK_SCALE(scale), 3);
 		gtk_range_set_update_policy(GTK_RANGE(scale), GTK_UPDATE_DISCONTINUOUS);
-		gtk_table_attach_defaults(GTK_TABLE(table), scale, 0, 4, 10, 12);
+		gtk_table_attach_defaults(GTK_TABLE(table), scale, 0, 4, 12, 13);
 	}
 
 	DEBUG_MSG("image_dialog, calling activate on location entry\n");
 	gtk_signal_emit_by_name(GTK_OBJECT(image_win->location), "activate");
 
-	gtk_table_attach_defaults(GTK_TABLE(table), bf_stock_ok_button(insert_image_html, image_win), 0, 2, 12, 13);
-	gtk_table_attach_defaults(GTK_TABLE(table), bf_stock_cancel_button(destroy_image_win, image_win), 2, 4, 12, 13);
+	but = bf_stock_ok_button(insert_image_html, image_win);
+	gtk_window_set_default(GTK_WINDOW(image_win->pixwin), but);
+	gtk_table_attach_defaults(GTK_TABLE(table), but , 0, 2, 13, 14);
+
+	gtk_table_attach_defaults(GTK_TABLE(table), bf_stock_cancel_button(destroy_image_win, image_win), 2, 4, 13, 14);
 	gtk_widget_show_all(image_win->pixwin);
 
 	gtk_widget_realize(GTK_WIDGET(image_win->pixwin));
 	gtk_window_set_transient_for(GTK_WINDOW(image_win->pixwin), GTK_WINDOW(main_v->main_window));
 
 	g_free(custom);
-	DEBUG_MSG("image_dialog, finished, image_win->replace_range=%p\n", image_win->replace_range);
+
+            /* switch colormap back to GdkRgb colormap */
+        gtk_widget_pop_visual ();
+        gtk_widget_pop_colormap ();
+        
+	/* DEBUG_MSG("image_dialog, finished, image_win->replace_range=%p\n", image_win->replace_range);*/
 }
 
 /************************************************************************/
@@ -326,21 +351,23 @@ static void insert_image_html(GtkWidget * widget, Timagewin * win)
 	tempstring = insert_integer_if_spin(win->border, cap("BORDER"), tempstring, NULL);
 	tempstring = insert_integer_if_spin(win->vspace, cap("VSPACE"), tempstring, NULL);
 	tempstring = insert_integer_if_spin(win->hspace, cap("HSPACE"), tempstring, NULL);
-	tempstring = insert_string_if_entry(win->alt, cap("ALT"), tempstring);
-	tempstring = insert_string_if_entry(GTK_COMBO(win->align)->entry, cap("ALIGN"), tempstring);
+	tempstring = insert_string_if_entry(win->custom, NULL, tempstring, NULL);
+	tempstring = insert_string_if_entry(win->alt, cap("ALT"), tempstring, (main_v->props.xhtml == 1 ? "No description" : NULL));
+	tempstring = insert_string_if_entry(win->name, cap("NAME"), tempstring, NULL);
+	tempstring = insert_string_if_entry(win->usemap, cap("USEMAP"), tempstring, NULL);
+	tempstring = insert_string_if_entry(GTK_COMBO(win->align)->entry, cap("ALIGN"), tempstring, NULL);
 	DEBUG_MSG("insert_image_html, 3, tempstring=%s\n", tempstring);
 	if (win->thumbnail) {
-		finalstring = g_strdup_printf("%s></%s>", tempstring, cap("A"));
+		finalstring = g_strdup_printf(main_v->props.xhtml == 1 ? "%s /></%s>" : "%s></%s>", tempstring, cap("A"));
 	} else {
-		finalstring = g_strdup_printf("%s>", tempstring);
+		finalstring = g_strdup_printf(main_v->props.xhtml == 1 ? "%s />" : "%s>", tempstring);
 	}
 	g_free(tempstring);
 
-	if (win->replace_range == NULL) {
-		insert_text(finalstring, "");
+	if (win->range.len == -1) {
+		insert_dbl_text(finalstring, "");
 	} else {
-		replace_text(finalstring, win->replace_range->pos, win->replace_range->len);
-		g_free(win->replace_range);
+		replace_text(finalstring, win->range.pos, win->range.len);
 	}
 
 	g_free(finalstring);
@@ -414,53 +441,108 @@ static void image_adjust_changed(GtkAdjustment * adj, Timagewin * image_win)
  * touch these and I will kill you Olivier 
  * <evil laugh> 
  */
+static void image_insert_dialogok_lcb(GtkWidget * widget, Thtml_diag *dg) {
+	gchar *thestring, *finalstring;
 
-void image_insert_dialog_cb(GtkWidget * widget, gpointer data)
-{
-	GtkWidget *dialog, *vbox, *label, *button;
+	thestring = g_strdup(cap("<IMG"));
+	thestring = insert_string_if_entry(dg->entry[0], cap("SRC"), thestring, NULL);
+	thestring = insert_integer_if_spin(dg->spin[0], cap("WIDTH"), thestring, dg->check[0]);
+	thestring = insert_integer_if_spin(dg->spin[1], cap("HEIGHT"), thestring, dg->check[1]);
+	thestring = insert_integer_if_spin(dg->spin[2], cap("BORDER"), thestring, NULL);
+	thestring = insert_integer_if_spin(dg->spin[3], cap("HSPACE"), thestring, NULL);
+	thestring = insert_integer_if_spin(dg->spin[4], cap("VSPACE"), thestring, NULL);
+	thestring = insert_string_if_entry(dg->entry[1], cap("NAME"), thestring, NULL);
+	thestring = insert_string_if_entry(dg->entry[2], cap("ALT"), thestring, NULL);
+	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->combo[0])->entry), cap("ALIGN"), thestring, NULL);
+	thestring = insert_string_if_entry(dg->entry[3], NULL, thestring, NULL);
 
-	dialog = gtk_window_new(GTK_WINDOW_DIALOG);
-	gtk_signal_connect(GTK_OBJECT(dialog), "delete_event", GTK_SIGNAL_FUNC(gtk_widget_destroy), (GtkWidget *) dialog);
-	gtk_signal_connect(GTK_OBJECT(dialog), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroy), (GtkWidget *) dialog);
-	gtk_widget_set_usize(GTK_WIDGET(dialog), 300, 53);
-	gtk_window_set_title(GTK_WINDOW(dialog), _("hmmmmmm...."));
-	gtk_window_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
-	vbox = gtk_vbox_new(FALSE, 0);
-	gtk_container_add(GTK_CONTAINER(dialog), vbox);
-	gtk_widget_show(vbox);
-	label = gtk_label_new(_("This version of Bluefish was not\ncompiled with imlib support. sorry."));
-	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
-	button = gtk_button_new_with_label(_("OK"));
-	gtk_signal_connect_object(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(gtk_object_destroy), (GtkObject *) dialog);
-	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
-	gtk_widget_show(button);
-	gtk_widget_show(dialog);
+	finalstring = g_strdup_printf(main_v->props.xhtml == 1 ? "%s />" : "%s>", thestring);
+	g_free(thestring);
+
+	if (dg->range.len == -1) {
+		insert_dbl_text(finalstring, NULL);
+	} else {
+		replace_text(finalstring, dg->range.pos, dg->range.len);
+	}
+
+	g_free(finalstring);
+	html_diag_destroy_cb(NULL, NULL, dg);	
+
+}
+void image_insert_dialog_cb(GtkWidget * widget, gpointer data) {
+	static gchar *tagitems[] = { "width", "height", "alt", "border", "src", "hspace", "vspace", "align", "name", "usemap", NULL };
+	gchar *tagvalues[11];
+	gchar *custom = NULL;
+	Thtml_diag *dg;
+	
+	GtkWidget *dgtable;
+
+	dg = html_diag_new(_("Insert image"));
+	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, widget, dg);
+
+	dgtable = gtk_table_new(7, 6, 0);
+	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
+	
+	dg->entry[0] = entry_with_text(tagvalues[4], 1024);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Image location")), 0, 1, 0, 1);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->entry[0], 1, 5, 0, 1);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), file_but_new(dg->entry[0], dg->dialog, 0), 5, 6, 0, 1);
+
+	dg->spin[0] = spinbut_with_value(NULL, 0, 500, 1.0, 5.0);
+	dg->check[0] = gtk_check_button_new_with_label("%");
+	parse_integer_for_dialog(tagvalues[0], dg->spin[0] , NULL, dg->check[0]);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Width")), 0, 1, 1, 2);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->spin[0], 1, 2, 1, 2);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->check[0], 2, 3, 1, 2);
+
+	dg->spin[1] = spinbut_with_value(NULL, 0, 500, 1.0, 5.0);
+	dg->check[1] = gtk_check_button_new_with_label("%");
+	parse_integer_for_dialog(tagvalues[1], dg->spin[1] , NULL, dg->check[1]);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Height")), 0, 1, 2, 3);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->spin[1], 1, 2, 2, 3);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->check[1], 2, 3, 2, 3);
+
+	dg->entry[3] = entry_with_text(tagvalues[9], 1024);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Usemap")), 0, 1, 3, 4);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->entry[3], 1, 3, 3, 4);
+
+	dg->entry[1] = entry_with_text(tagvalues[8], 1024);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Name")), 0, 1, 4, 5);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->entry[1], 1, 3, 4, 5);
+
+	dg->entry[2] = entry_with_text(tagvalues[2], 1024);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Alternate text")), 0, 1, 5, 6);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->entry[2], 1, 6, 5, 6);
+
+	dg->entry[3] = entry_with_text(custom, 1024);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Custom")), 0, 1, 6, 7);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->entry[3], 1, 6, 6, 7);
+
+	dg->spin[3] = spinbut_with_value(tagvalues[5], 0, 500, 1.0, 5.0);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Hspace")), 3, 4, 1, 2);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->spin[3], 4, 6, 1, 2);
+
+	dg->spin[4] = spinbut_with_value(tagvalues[6], 0, 500, 1.0, 5.0);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Vspace")), 3, 4, 2, 3);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->spin[4], 4, 6, 2, 3);
+
+	dg->combo[0] = combo_with_popdown_sized(tagvalues[7], positionlist, 1, 90);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Align")), 3, 4, 3, 4);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->combo[0], 4, 6, 3, 4);
+
+	dg->spin[2] = spinbut_with_value(tagvalues[3], 0, 500, 1.0, 5.0);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable),gtk_label_new(_("Border")) , 3, 4, 4, 5);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->spin[2], 4, 6, 4, 5);
+
+	html_diag_finish(dg, image_insert_dialogok_lcb);
+	
+	if (custom)	g_free(custom);
 }
 
 /************************************************************************/
 
-void image_thumbnail_dialog_cb(GtkWidget * widget, gpointer data)
-{
-	GtkWidget *dialog, *vbox, *label, *button;
-
-	dialog = gtk_window_new(GTK_WINDOW_DIALOG);
-	gtk_signal_connect(GTK_OBJECT(dialog), "delete_event", GTK_SIGNAL_FUNC(gtk_widget_destroy), (GtkWidget *) dialog);
-	gtk_signal_connect(GTK_OBJECT(dialog), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroy), (GtkWidget *) dialog);
-	gtk_widget_set_usize(GTK_WIDGET(dialog), 300, 53);
-	gtk_window_set_title(GTK_WINDOW(dialog), _("hmmmmmm...."));
-	gtk_window_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
-	vbox = gtk_vbox_new(FALSE, 0);
-	gtk_container_add(GTK_CONTAINER(dialog), vbox);
-	gtk_widget_show(vbox);
-	label = gtk_label_new(_("This version of Bluefish was not\ncompiled with imlib support. sorry."));
-	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
-	button = gtk_button_new_with_label("OK");
-	gtk_signal_connect_object(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(gtk_object_destroy), (GtkObject *) dialog);
-	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
-	gtk_widget_show(button);
-	gtk_widget_show(dialog);
+void image_thumbnail_dialog_cb(GtkWidget * widget, gpointer data) {
+	error_dialog(_("Insert image as thumbnail"), _("Bluefish was compiled without\nImlib support. This function is\ntherefore not available"));
 }
 
 /************************************************************************/
