@@ -24,8 +24,7 @@
 #include <glib.h>
 #include <gtk/gtkwidget.h>
 
-void image_insert_dialog (GtkWidget * widget, gpointer data);
-void image_thumbnail_dialog (GtkWidget * widget, gpointer data);
-
-#endif	/* __IMAGES_H_ */
+void image_insert_dialog_cb(GtkWidget * widget, gpointer data);
+void image_thumbnail_dialog_cb(GtkWidget * widget, gpointer data);
 
+#endif							/* __IMAGES_H_ */
