@@ -18,7 +18,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-
 #include "default_include.h"
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -32,9 +31,9 @@
 #include <locale.h>
 
 #include "bluefish.h"
+#include "bf_lib.h"
 #include "init.h"
 #include "stringlist.h"
-#include "highlight.h"
 #include "filter.h"
 
 #define DIR_MODE (S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)	/* same as 0755 */
@@ -169,19 +168,141 @@ gint check_files_links_directories()
 	DEBUG_MSG("check_directories, homedir=%s\n", homedir);
 	chk_dir(strncat(strncpy(name, homedir, PATH_MAX), "/.bluefish/", PATH_MAX));
 	chk_dir(strncat(strncpy(name, homedir, PATH_MAX), "/.bluefish/projects/", PATH_MAX));
-	chk_dir(strncat(strncpy(name, homedir, PATH_MAX), "/.bluefish/menu", PATH_MAX));
 	chk_link(strncat(strncpy(name, homedir, PATH_MAX), "/.bluefish/php3_functions", PATH_MAX),
-			 strncat(strncpy(name2, PKGDATADIR, PATH_MAX), "php3_functions", PATH_MAX));
+			 strncat(strncpy(name2, PKGDATADIR, PATH_MAX), "/php3_functions", PATH_MAX));
 	chk_link(strncat(strncpy(name, homedir, PATH_MAX), "/.bluefish/ssi_functions", PATH_MAX),
-			 strncat(strncpy(name2, PKGDATADIR, PATH_MAX), "ssi_functions", PATH_MAX));
+			 strncat(strncpy(name2, PKGDATADIR, PATH_MAX), "/ssi_functions", PATH_MAX));
 	chk_link(strncat(strncpy(name, homedir, PATH_MAX), "/.bluefish/rxml_functions", PATH_MAX),
-			 strncat(strncpy(name2, PKGDATADIR, PATH_MAX), "rxml_functions", PATH_MAX));
+			 strncat(strncpy(name2, PKGDATADIR, PATH_MAX), "/rxml_functions", PATH_MAX));
+ chk_link(strncat(strncpy(name, homedir, PATH_MAX), "/.bluefish/php4_functions", PATH_MAX),
+			 strncat(strncpy(name2, PKGDATADIR, PATH_MAX), "/php4_functions", PATH_MAX));
 	chk_file(strncat(strncpy(name, homedir, PATH_MAX), "/.bluefish/rcfile", PATH_MAX),RCFILE_DEFAULTS);
 	g_free(name);
 	g_free(name2);
 	return 1;
 }
 
+/********************************************************************/
+
+static gchar *array_to_string(gchar **array, gchar delimiter) {
+
+	gchar **tmp, *escaped1, *finalstring;
+	gint newsize=1;
+	DEBUG_MSG("array_to_string, started\n");
+	finalstring = g_malloc0(newsize);
+	tmp = array;
+	while(*tmp) {
+		DEBUG_MSG("array_to_string, *tmp = %s\n", *tmp);
+		escaped1 = escapestring(*tmp, delimiter);
+		newsize += strlen(escaped1)+1;
+		finalstring = g_realloc(finalstring, newsize);
+		strcat(finalstring, escaped1);
+		finalstring[newsize-2] = delimiter;
+		finalstring[newsize-1] = '\0';
+		g_free(escaped1);
+		tmp++;
+	}	
+	DEBUG_MSG("array_to_string, finalstring = %s\n", finalstring);
+	return finalstring;
+}
+#define ARRAYBLOCKSIZE 4
+#define BUFBLOCKSIZE 30
+
+gchar *my_strdup(gchar *original) {
+	gchar *copy;
+	gint len = strlen(original);
+
+	DEBUG_MSG("my_strdup, len=%d, original=%p\n", len, original);
+	copy = g_malloc(len+1);
+	DEBUG_MSG("my_strdup, copy=%p\n", copy);
+	strncpy(copy, original, len);
+	copy[len+1] = '\0';
+	return copy;
+}
+
+gchar **string_to_array(gchar *string, gchar delimiter) {
+	gchar **array;
+	gchar *tmpchar, *tmpchar2;
+	gchar *newstring;
+	gint count=0;
+	gint newstringsize;
+	gint arraycount=0, arraysize;
+
+	newstringsize = BUFBLOCKSIZE;
+	newstring = g_malloc(newstringsize * sizeof(char));
+	
+	arraysize = ARRAYBLOCKSIZE;
+	array = g_malloc(arraysize * sizeof(char *));
+	DEBUG_MSG("string_to_array, started, array=%p\n", array);	
+	
+	tmpchar = string;
+	while (*tmpchar != '\0') {
+		DEBUG_MSG("string_to_array, count=%d, newstring(%p)\n", count, newstring);
+		if (*tmpchar == '\\') {
+			tmpchar2 = tmpchar;
+			tmpchar2++;
+			switch (*tmpchar2) {
+			case '\0':
+				newstring[count] = '\\';
+			break;
+			case '\\':
+				newstring[count] = '\\';
+				tmpchar++;
+			break;
+			case 'n':
+				newstring[count] = '\n';
+				tmpchar++;
+			break;
+			default:
+				if (*tmpchar2 == delimiter) {
+					newstring[count] = delimiter;
+					tmpchar++;
+				} else {
+					DEBUG_MSG("string_to_array, weird, an unescaped backslash ?\n");
+					newstring[count] = '\\';
+				}
+			break;
+			}
+		} else if (*tmpchar == delimiter) {
+			newstring[count] = '\0';  /* end of the current newstring */
+			DEBUG_MSG("string_to_array, newstring(%p)=%s\n", newstring, newstring);
+			array[arraycount] = g_strdup(newstring);
+			DEBUG_MSG("string_to_array, found delimiter, arraycount=%d, result(%p)=%s\n",arraycount, array[arraycount], array[arraycount]);
+			arraycount++;
+			if (arraycount == arraysize-2) { /* we need 1 spare entry in the array */
+				arraysize += ARRAYBLOCKSIZE;  /* and arraysize starts at 1, arraycount at 0 */
+				DEBUG_MSG("string_to_array, arraycount=%d, about to increase arraysize to %d, sizeof(array(%p))=%d\n", arraycount, arraysize, array, sizeof(&array));
+				array = g_realloc(array, arraysize * sizeof(char *));
+				DEBUG_MSG("string_to_array, arraysize=%d, array(%p), sizeof(array)=%d\n", arraysize, array, sizeof(&array));
+			}
+			count = -1;
+		} else {
+			newstring[count] = *tmpchar;
+		}
+		tmpchar++;
+		count++;
+		if (count == newstringsize-2) {
+			newstringsize += BUFBLOCKSIZE;
+			DEBUG_MSG("string_to_array, about to increase newstring(%p) to %d bytes\n", newstring, newstringsize);
+			newstring = g_realloc(newstring, newstringsize * sizeof(char));
+			DEBUG_MSG("string_to_array, newstringsize=%d, sizeof(newstring(%p))=%d\n", newstringsize, newstring, sizeof(newstring));
+		}
+	}
+	
+	if (count > 0) {
+		newstring[count] = '\0';
+		array[arraycount] = g_strdup(newstring);
+		DEBUG_MSG("string_to_array, last array entry, arraycount=%d, result(%p)=%s\n",arraycount, array[arraycount],array[arraycount]);
+	} else {
+		array[arraycount] = NULL;
+	}
+	array[arraycount+1] = NULL; /* since we have 1 spare entry in the array this is safe to do*/
+	DEBUG_MSG("string_to_array, returning %p\n", array);
+	g_free(newstring);
+	return array;
+}
+/********************************************************************/
+
 /*this should add 1 empty entry to the configuration list */
 GList *make_config_list_item(GList * config_list, void *pointer_to_var, unsigned char type_of_var, gchar * name_of_var)
 {
@@ -210,12 +331,24 @@ static void init_prop_string(GList ** config_list, void *pointer_to_var, gchar *
 	DEBUG_MSG("init_prop_string, name_of_var=%s, default_value=%s\n", name_of_var, default_value);
 }
 
-static void init_prop_list(GList ** config_list, void *pointer_to_var, gchar * name_of_var)
+static void init_prop_string_with_escape(GList ** config_list, void *pointer_to_var, gchar * name_of_var, gchar * default_value)
+{
+	*config_list = make_config_list_item(*config_list, pointer_to_var, 'e', name_of_var);
+	*(gchar **) pointer_to_var = unescapestring(default_value);
+	DEBUG_MSG("init_prop_string, name_of_var=%s, default_value=%s\n", name_of_var, default_value);
+}
+
+/* static void init_prop_list(GList ** config_list, void *pointer_to_var, gchar * name_of_var)
 {
 	*config_list = make_config_list_item(*config_list, pointer_to_var, 'l', name_of_var);
 	pointer_to_var = NULL;
-}
+} */
 
+static void init_prop_arraylist(GList ** config_list, void *pointer_to_var, gchar * name_of_var)
+{
+	*config_list = make_config_list_item(*config_list, pointer_to_var, 'a', name_of_var);
+	pointer_to_var = NULL;
+}
 
 
 gint save_config_file(GList * config_list, gchar * filename)
@@ -258,6 +391,20 @@ gint save_config_file(GList * config_list, gchar * filename)
 				rclist = g_list_append(rclist, tmpstring);
 			}
 			break;
+		case 'e':
+			DEBUG_MSG("save_config_file, converting \"%p\"\n", tmpitem);
+			DEBUG_MSG("save_config_file, converting \"%s %s\"\n", tmpitem->identifier, (gchar *) * (void **) tmpitem->pointer);
+			if (*(void **) tmpitem->pointer) {
+				tmpstring2 = escapestring((gchar*)*(void**)tmpitem->pointer, '\0');
+				tmpstring = g_strdup_printf("%s %s", tmpitem->identifier, tmpstring2);
+				CHECK_MEMORY(tmpstring == NULL, "save_config_file:tmpstring:2");
+
+				DEBUG_MSG("save_config_file, adding %s\n", tmpstring);
+
+				rclist = g_list_append(rclist, tmpstring);
+				g_free(tmpstring2);
+			}
+			break;
 		case 'l':
 			DEBUG_MSG("save_config_file, tmpitem(%p), &tmpitem=%p\n", tmpitem, &tmpitem);
 			tmplist2 = g_list_last((GList *) * (void **) tmpitem->pointer);
@@ -273,6 +420,20 @@ gint save_config_file(GList * config_list, gchar * filename)
 				tmplist2 = g_list_previous(tmplist2);
 			}
 			break;
+		case 'a':
+			DEBUG_MSG("save_config_file, tmpitem(%p), &tmpitem=%p\n", tmpitem, &tmpitem);
+			tmplist2 = g_list_last((GList *) * (void **) tmpitem->pointer);
+			DEBUG_MSG("save_config_file, the tmplist2(%p)\n", tmplist2);
+			while (tmplist2 != NULL) {
+				tmpstring2 = array_to_string((char **) tmplist2->data, ':');
+				tmpstring = g_strdup_printf("%s %s", tmpitem->identifier, tmpstring2);
+				CHECK_MEMORY(tmpstring == NULL, "save_config_file:tmpstring:3");
+				DEBUG_MSG("save_config_file, tmpstring(%p)=%s\n", tmpstring, tmpstring);
+				rclist = g_list_append(rclist, tmpstring);
+				tmplist2 = g_list_previous(tmplist2);
+				g_free(tmpstring2);
+			}
+			break;
 		default:
 			break;
 		}
@@ -291,6 +452,7 @@ gint save_config_file(GList * config_list, gchar * filename)
 gint parse_config_file(GList * config_list, gchar * filename)
 {
 	gchar *tmpstring = NULL, *tmpstring2;
+	gchar **tmparray;
 	GList *rclist, *tmplist, *tmplist2;
 	Tconfig_list_item *tmpitem;
 
@@ -304,14 +466,15 @@ gint parse_config_file(GList * config_list, gchar * filename)
 	while (tmplist != NULL) {
 		tmpitem = (Tconfig_list_item *) tmplist->data;
 		DEBUG_MSG("parse_config_file, type=%c, identifier=%s\n", tmpitem->type, tmpitem->identifier);
-		if (tmpitem->type == 'l') {
+		if (tmpitem->type == 'l' || tmpitem->type == 'a') {
+			DEBUG_MSG("parse_config_file, freeing list before filling it\n");
 			free_stringlist((GList *) * (void **) tmpitem->pointer);
 			(GList *) * (void **) tmpitem->pointer = NULL;
 		}
 		DEBUG_MSG("parse_config_file, type=%c, identifier=%s\n", tmpitem->type, tmpitem->identifier);
 		tmplist = g_list_next(tmplist);
 	}
-	DEBUG_MSG("parse_config_file, all the type 'l' have been emptied\n");
+	DEBUG_MSG("parse_config_file, all the type 'l' and 'a' have been emptied\n");
 	DEBUG_MSG("parse_config_file, length rclist=%d\n", g_list_length(rclist));
 /* And now for parsing every line in the config file, first check if there is a valid identifier at the start. */
 	tmplist = g_list_first(rclist);
@@ -325,6 +488,11 @@ gint parse_config_file(GList * config_list, gchar * filename)
 			tmplist2 = g_list_first(config_list);
 			while (tmplist2) {
 				tmpitem = (Tconfig_list_item *) tmplist2->data;
+#ifdef DEVELOPMENT
+				if (!tmpitem || !tmpitem->identifier || !tmpstring) {
+					g_print("WARNING: almost a problem!\n");
+				}
+#endif
 				if (g_strncasecmp(tmpitem->identifier, tmpstring, strlen(tmpitem->identifier)) == 0) {
 					/* we have found the correct identifier */
 					DEBUG_MSG("parse_config_file, identifier=%s, string=%s\n", tmpitem->identifier, tmpstring);
@@ -342,12 +510,24 @@ gint parse_config_file(GList * config_list, gchar * filename)
 						(char *) *(void **) tmpitem->pointer = realloc((char *) *(void **) tmpitem->pointer, strlen(tmpstring) + 1);
 						strcpy((char *) *(void **) tmpitem->pointer, tmpstring);
 						break;
+					case 'e':
+						tmpstring2 = unescapestring(tmpstring);
+						(char *) *(void **) tmpitem->pointer = realloc((char *) *(void **) tmpitem->pointer, strlen(tmpstring2) + 1);
+						strcpy((char *) *(void **) tmpitem->pointer, tmpstring2);
+						g_free(tmpstring2);
+						break;
 					case 'l':
 						tmpstring2 = g_strdup(tmpstring);
 						CHECK_MEMORY(tmpstring2 == NULL, "parse_config_file:tmpstring2");
 						(GList *) * (void **) tmpitem->pointer = g_list_prepend((GList *) * (void **) tmpitem->pointer, tmpstring2);
 						DEBUG_MSG("parse_config_file, *(void **)tmpitem->pointer=%p\n", *(void **) tmpitem->pointer);
 						break;
+					case 'a':
+						tmparray = string_to_array(tmpstring, ':');
+						CHECK_MEMORY(tmparray == NULL, "parse_config_file:tmparray");
+						(GList *) * (void **) tmpitem->pointer = g_list_prepend((GList *) * (void **) tmpitem->pointer, tmparray);
+						DEBUG_MSG("parse_config_file, *(void **)tmpitem->pointer=%p\n", *(void **) tmpitem->pointer);
+						break;
 					default:
 						break;
 					}
@@ -358,6 +538,7 @@ gint parse_config_file(GList * config_list, gchar * filename)
 		}
 		tmplist = g_list_next(tmplist);
 	}
+	DEBUG_MSG("parse_config_file, parsed all entries, freeing list read from file\n");	
 	free_stringlist(rclist);
 	return 1;
 }
@@ -462,7 +643,11 @@ void make_external_menu_entries(gint which_external)
 
 	tmplist = g_list_first(config_stringlist);
 	while (tmplist) {
-		splittedstring = g_strsplit((gchar *) tmplist->data, "/", 2);
+		splittedstring = (gchar **) tmplist->data;
+		if (!splittedstring[0] || !splittedstring[1]) {
+			tmplist = g_list_next(tmplist);
+			continue;
+		}
 		g_strstrip(splittedstring[0]);
 		path = g_strconcat(menupath, splittedstring[0], NULL);
 		exmen = g_malloc0(sizeof(Texternal_menu));
@@ -475,7 +660,6 @@ void make_external_menu_entries(gint which_external)
 
 		*commandlist = g_list_append(*commandlist, exmen);
 
-		g_strfreev(splittedstring);
 		count++;
 		tmplist = g_list_next(tmplist);
 	}
@@ -490,53 +674,87 @@ void make_external_menu_entries(gint which_external)
  */
 static GList *main_v_props_init(GList * config_rc)
 {
+	init_prop_integer(&config_rc, &main_v->props.v_html_tb, "view_html_toolbar:", 1);
+	init_prop_integer(&config_rc, &main_v->props.v_custom_tb, "view_custom_toolbar:", 1);
+	init_prop_integer(&config_rc, &main_v->props.v_main_tb, "view_main_toolbar:", 1);
+	init_prop_integer(&config_rc, &main_v->props.transient_htdialogs, "transient_htdialogs:", 0);
 	init_prop_string(&config_rc, &main_v->props.cfg_browser_cline, "browser_command:", NETSCAPE_COMMAND);
 	init_prop_string(&config_rc, &main_v->props.cfg_editor_font, "editor_font:", "fixed,*");
 	init_prop_integer(&config_rc, &main_v->props.cfg_editor_tabwidth, "editor_tab_width:", 8);
 	init_prop_string(&config_rc, &main_v->props.cfg_tab_font, "notebook_tab_font:", "fixed,*");
 	init_prop_string(&config_rc, &main_v->props.cfg_tab_pos, "notebook_tab_position:", "bottom");
 	init_prop_string(&config_rc, &main_v->props.cfg_weblint_cline, "weblint_command:", WEBLINT_COMMAND);
-	init_prop_integer(&config_rc, &main_v->props.v_html_tb, "view_html_toolbar:", 1);
-	init_prop_integer(&config_rc, &main_v->props.v_custom_tb, "view_custom_toolbar:", 0);
-	init_prop_integer(&config_rc, &main_v->props.v_main_tb, "view_main_toolbar:", 1);
 	init_prop_integer(&config_rc, &main_v->props.main_window_h, "main_window_height:", 400);
 	init_prop_integer(&config_rc, &main_v->props.main_window_w, "main_window_width:", 600);
+	init_prop_integer(&config_rc, &main_v->props.main_filelist_size, "main_filelist_size:", 150);
+	init_prop_integer(&config_rc, &main_v->props.max_recent_files, "max_recent_files:", 10);
+	init_prop_integer(&config_rc, &main_v->props.max_dir_history, "max_dir_history:", 5);
+	init_prop_string(&config_rc, &main_v->props.filelist_filter, "filelist_filter:", _("all web files"));
+	init_prop_string(&config_rc, &main_v->props.filebrowser_filter_image, "filebrowser_filter_image:", ".gif:.png:.jpg:.jpeg");
+	init_prop_string(&config_rc, &main_v->props.filebrowser_filter_php, "filebrowser_filter_php:", ".php:.phtml:.php3:.php4:.phps");
+	init_prop_string(&config_rc, &main_v->props.filebrowser_filter_html, "filebrowser_filter_html:", ".htm:.html:.shtml");
+	init_prop_string(&config_rc, &main_v->props.filebrowser_filter_jscript, "filebrowser_filter_jscript:", ".js");
+	init_prop_string(&config_rc, &main_v->props.filebrowser_filter_css, "filebrowser_filter_css:", ".css");
+	init_prop_string(&config_rc, &main_v->props.filebrowser_filter_other, "filebrowser_filter_other:", ".htdocs:.asp:.pl:.cgi:.jsp");
 	init_prop_string(&config_rc, &main_v->props.cfg_thumbnailstring, "thumbnail_string:", "_thumbnail");
 	init_prop_string(&config_rc, &main_v->props.cfg_thumbnailtype, "thumbnail_type:", "jpg");
 	init_prop_integer(&config_rc, &main_v->props.full_p, "closing_paragraph_tag:", 1);
-	init_prop_integer(&config_rc, &main_v->props.full_li, "closing_list_item_tag:", 0);
-	init_prop_integer(&config_rc, &main_v->props.lowercase_tags, "lowercase_tags:", 0);
+	init_prop_integer(&config_rc, &main_v->props.full_li, "closing_list_item_tag:", 1);
+	init_prop_integer(&config_rc, &main_v->props.allow_css, "allow_the_use_of_css:", 1);
+	init_prop_integer(&config_rc, &main_v->props.allow_dep, "allow_the_use_of_font:", 1);
+	init_prop_integer(&config_rc, &main_v->props.format_by_context, "format_by_context:", 1);
+	init_prop_integer(&config_rc, &main_v->props.xhtml, "use_xhtml:", 0);
+	init_prop_integer(&config_rc, &main_v->props.allow_ruby, "allow_the_use_of_ruby:", 0);
+	init_prop_integer(&config_rc, &main_v->props.allow_h4, "allow_the_use_of_html4:", 1);
+	init_prop_integer(&config_rc, &main_v->props.allow_frames, "allow_the_use_of_frames:", 1);
+	init_prop_integer(&config_rc, &main_v->props.force_dtd, "force_an_dtd:", 0);
+	init_prop_integer(&config_rc, &main_v->props.dtd_url, "url_in_dtd:", 0);
+	init_prop_integer(&config_rc, &main_v->props.xml_start, "xml_starting_line:", 0);
+	init_prop_integer(&config_rc, &main_v->props.lowercase_tags, "lowercase_tags:", 1);
 	init_prop_integer(&config_rc, &main_v->props.word_wrap, "word_wrap:", 0);
+	init_prop_integer(&config_rc, &main_v->props.autoindent, "autoindent:", 1);
 	init_prop_integer(&config_rc, &main_v->props.line_wrap, "line_wrap:", 1);
 	init_prop_integer(&config_rc, &main_v->props.fontset, "fontset:", 0);
-	init_prop_integer(&config_rc, &main_v->props.link_management, "link_management:", 0);
+	init_prop_integer(&config_rc, &main_v->props.force_def_style, "force_white_background:", 1);
+	init_prop_integer(&config_rc, &main_v->props.drop_at_drop_pos, "drop_at_drop_position:", 0);
+	init_prop_integer(&config_rc, &main_v->props.link_management, "link_management:", 1);
 	init_prop_integer(&config_rc, &main_v->props.defaulthighlight, "defaulthighlight:", 1);
 	init_prop_integer(&config_rc, &main_v->props.cont_highlight_full, "cont_highlight_full:", 0);
-	init_prop_integer(&config_rc, &main_v->props.cont_highlight_update, "continuous_highlight_update:", 0);
-	init_prop_string(&config_rc, &main_v->props.default_charset, "default_charset:", "iso8859-1");
-	init_prop_list(&config_rc, &main_v->props.external_filters, "external_filters:");
-	init_prop_list(&config_rc, &main_v->props.external_commands, "external_commands:");
-	init_prop_list(&config_rc, &main_v->props.syntax_configstrings, "highlight_patterns:");
-	
-	
+	init_prop_integer(&config_rc, &main_v->props.cont_highlight_update, "continuous_highlight_update:", 1);
+	init_prop_string(&config_rc, &main_v->props.html_ver, "used_html_version:", "HTML 4.0 Transitional");
+	init_prop_arraylist(&config_rc, &main_v->props.external_filters, "external_filters:");
+	init_prop_arraylist(&config_rc, &main_v->props.external_commands, "external_commands:");
+	init_prop_arraylist(&config_rc, &main_v->props.syntax_configstrings, "highlight_patterns:");
+	init_prop_arraylist(&config_rc, &main_v->props.cust_menu, "custom_menu_ccs:");
 
 	/* spell checker options: */
-	
 #ifdef WITH_SPC
 	init_prop_string (&config_rc, &main_v->props.cfg_spc_cline,"spell_checker_command:",ISPELL_COMMAND);
 	init_prop_string (&config_rc, &main_v->props.cfg_spc_lang ,"spell_checker_language:","default");
-        init_prop_integer (&config_rc, &main_v->props.spc_accept_compound,"spell_checker_accept_compound:",0);
+	init_prop_integer (&config_rc, &main_v->props.spc_accept_compound,"spell_checker_accept_compound:",0);
 	init_prop_integer (&config_rc, &main_v->props.spc_use_esc_chars,"spell_checker_use_esc_chars:",0);
-        init_prop_string  (&config_rc, &main_v->props.spc_esc_chars,"spell_checker_esc_chars:",NULL);
-        init_prop_integer (&config_rc, &main_v->props.spc_use_pers_dict,"spell_checker_use,pers_dict:",0);
-        init_prop_string  (&config_rc, &main_v->props.spc_use_pers_dict,"spell_checker_pers_dict:",NULL);
+	init_prop_string  (&config_rc, &main_v->props.spc_esc_chars,"spell_checker_esc_chars:",NULL);
+	init_prop_integer (&config_rc, &main_v->props.spc_use_pers_dict,"spell_checker_use,pers_dict:",0);
+	init_prop_string  (&config_rc, &main_v->props.spc_use_pers_dict,"spell_checker_pers_dict:",NULL);
+#endif
+	init_prop_string  (&config_rc, &main_v->props.backup_filestring,"backup_filestring:","~");
+	init_prop_integer (&config_rc, &main_v->props.backup_file,"backup_file:",1);
+	init_prop_integer (&config_rc, &main_v->props.allow_multi_instances,"allow_multi_instances:",0);
+
+	/* key conversion */
+	init_prop_integer (&config_rc, &main_v->props.conv_ctrl_enter,"conv_ctrl_enter:",0);
+	init_prop_integer (&config_rc, &main_v->props.conv_shift_enter,"conv_shift_enter:",0);
+	init_prop_string_with_escape (&config_rc, &main_v->props.shift_enter_text,"shift_enter_text:","<br>\\n");
+	init_prop_string_with_escape (&config_rc, &main_v->props.ctrl_enter_text,"ctrl_enter_text:","\\n</p>\\n<p>");
+	init_prop_integer (&config_rc, &main_v->props.conv_special_char,"conv_special_char:",0);
+#ifdef WITH_MSG_QUEUE
+	init_prop_integer (&config_rc, &main_v->props.open_in_running_bluefish,"open_in_running_bluefish:",1);
 #endif
 	return config_rc;
-
 }
 
 /*
- * Function: parse_config_file
+ * Function: parse_bluefish_rcfile
  * Arguments:
  *      void
  * Return value:
@@ -563,20 +781,6 @@ void parse_bluefish_rcfile(void)
 	CHECK_MEMORY(filename == NULL, "parse_config_file:filename");
 	parse_config_file(config_rc, filename);
 	g_free(filename);
-
-	/* make syntax highlighting list ready */
-	make_syntax_struct_list();
-
-/* initialise positionlist */
-	positionlist = g_list_append(positionlist, "top");
-	positionlist = g_list_append(positionlist, "middle");
-	positionlist = g_list_append(positionlist, "bottom");
-	positionlist = g_list_append(positionlist, "right");
-	positionlist = g_list_append(positionlist, "left");
-	positionlist = g_list_append(positionlist, "texttop");
-	positionlist = g_list_append(positionlist, "baseline");
-	positionlist = g_list_append(positionlist, "absmiddle");
-	positionlist = g_list_append(positionlist, "absbottom");
 };
 
 /*
