@@ -41,20 +41,21 @@ typedef struct {
  */
 extern GList *positionlist;
 
+/* these are commandline possibilities */
+extern int root_override;
+extern gchar *load_projectname;
+extern GList *load_filenames;
+
 /*
  * these are the functions in init.c that are used to parse config files
  */
-
-gint chk_dir(gchar * name);
-gint chk_link(gchar * src_name, gchar * dst_name);
-gint chk_file(gchar * name, gchar * content);
-gint check_directories();
+gint check_files_links_directories();
 GList *make_config_list_item(GList * config_list, void *pointer_to_var, unsigned char type_of_var, gchar * name_of_var);
 gint save_config_file(GList * config_list, gchar * filename);
 gint parse_config_file(GList * config_list, gchar * filename);
-void parse_config_files();
-void save_config_files();
 void make_external_menu_entries(gint which_external);
-gint rcfile_init();
+void parse_bluefish_rcfile(void);
+void save_config_files(void);
+int parse_commandline(int argc, char **argv);
 
 #endif							/* __INIT_H_ */
