@@ -2,7 +2,7 @@
  * menu.h - uhh, duh.
  *
  * Copyright (C) 1998 Olivier Sessink and Chris Mazuc
- *
+ * Copyright (C) 1999-2000 Olivier Sessink
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -22,19 +22,25 @@
 #define __MENU_H_
 
 #include <locale.h>
+#ifdef ENABLE_NLS
 #include <libintl.h>
-
 #include "intl.h"
+#endif /* ENABLE_NLS */
 
 void save_menu_shortcuts();
 void load_menu_shortcuts();
 void get_main_menu(GtkWidget * window, GtkWidget ** menubar);
-void show_cust_menubar_cb(GtkWidget * w, gpointer data);
+void make_cust_menubar(GtkWidget *cust_handle_box);
+
+
+extern void open_recent_list();
+extern void add_to_recent_list(gchar *filename);
+
 #ifdef ENABLE_NLS
 gchar *menu_translate(const gchar * path, gpointer data);
 #else
 #define menu_translate NULL
-#endif
+#endif /* ENABLE_NLS */
 
 
 #endif							/* __MENU_H_ */
