@@ -22,130 +22,144 @@
 
 #include <locale.h>
 
-#include "pixmaps/new.xpm"
-#include "pixmaps/close.xpm"
-#include "pixmaps/copy.xpm"
-#include "pixmaps/cut.xpm"
-#include "pixmaps/help.xpm"
-#include "pixmaps/yes.xpm"
-#include "pixmaps/no.xpm"
-#include "pixmaps/ok.xpm"
-#include "pixmaps/cancel.xpm"
-#include "pixmaps/open.xpm"
-#include "pixmaps/open_small.xpm"
-#include "pixmaps/paste.xpm"
-#include "pixmaps/preferences.xpm"
-#include "pixmaps/print.xpm"
-#include "pixmaps/redo.xpm"
-#include "pixmaps/save.xpm"
-#include "pixmaps/save_as.xpm"
-#include "pixmaps/search-replace.xpm"
-#include "pixmaps/search.xpm"
-#include "pixmaps/spellcheck.xpm"
-#include "pixmaps/undo.xpm"
-#include "pixmaps/quick_start.xpm"
-#include "pixmaps/body.xpm"
-#include "pixmaps/bold.xpm"
-#include "pixmaps/italic.xpm"
-#include "pixmaps/underline.xpm"
-#include "pixmaps/strikeout.xpm"
-#include "pixmaps/paragraph.xpm"
-#include "pixmaps/break.xpm"
-#include "pixmaps/nbsp.xpm"
-#include "pixmaps/anchor.xpm"
-#include "pixmaps/image.xpm"
-#include "pixmaps/thumbnail.xpm"
-#include "pixmaps/hrule.xpm"
-#include "pixmaps/center.xpm"
-#include "pixmaps/right.xpm"
-#include "pixmaps/comment.xpm"
-#include "pixmaps/email.xpm"
-#include "pixmaps/font.xpm"
-#include "pixmaps/font_base.xpm"
-#include "pixmaps/fontp1.xpm"
-#include "pixmaps/fontm1.xpm"
-#include "pixmaps/fontpre.xpm"
-#include "pixmaps/fontsub.xpm"
-#include "pixmaps/fontsuper.xpm"
-#include "pixmaps/fonth1.xpm"
-#include "pixmaps/fonth2.xpm"
-#include "pixmaps/fonth3.xpm"
-#include "pixmaps/fonth4.xpm"
-#include "pixmaps/fonth5.xpm"
-#include "pixmaps/fonth6.xpm"
-#include "pixmaps/tablewhiz.xpm"
-#include "pixmaps/table.xpm"
-#include "pixmaps/table-tr.xpm"
-#include "pixmaps/table-th.xpm"
-#include "pixmaps/table-td.xpm"
-#include "pixmaps/table2.xpm"
-#include "pixmaps/table-tr2.xpm"
-#include "pixmaps/table-th2.xpm"
-#include "pixmaps/table-td2.xpm"
-#include "pixmaps/table-capt.xpm"
-#include "pixmaps/framewhiz.xpm"
-#include "pixmaps/frameset.xpm"
-#include "pixmaps/frame.xpm"
-#include "pixmaps/frameset2.xpm"
-#include "pixmaps/frame2.xpm"
-#include "pixmaps/frame_no.xpm"
-#include "pixmaps/frame_base.xpm"
-#include "pixmaps/form.xpm"
-#include "pixmaps/form_submit.xpm"
-#include "pixmaps/form_reset.xpm"
-#include "pixmaps/form_text.xpm"
-#include "pixmaps/form_hidden.xpm"
-#include "pixmaps/form_textarea.xpm"
-#include "pixmaps/form_radio.xpm"
-#include "pixmaps/form_check.xpm"
-#include "pixmaps/form_select.xpm"
-#include "pixmaps/list.xpm"
-#include "pixmaps/list_ul.xpm"
-#include "pixmaps/list_ol.xpm"
-#include "pixmaps/list_li.xpm"
-#include "pixmaps/list_dl.xpm"
-#include "pixmaps/list_dt.xpm"
-#include "pixmaps/list_dd.xpm"
-#include "pixmaps/list_menu.xpm"
-#include "pixmaps/image_map.xpm"
-#include "pixmaps/meta.xpm"
-#include "pixmaps/embed.xpm"
-#include "pixmaps/netscape.xpm"
-#include "pixmaps/color_select.xpm"
-#include "pixmaps/color_select_small.xpm"
-#include "pixmaps/time.xpm"
-#include "pixmaps/nothing.xpm"
-#include "pixmaps/php3.xpm"
-#include "pixmaps/ssi.xpm"
-#include "pixmaps/rxml.xpm"
-#include "pixmaps/breakall.xpm"
-#include "pixmaps/dtd.xpm"
-#include "pixmaps/head.xpm"
-#include "pixmaps/cssnewstyle.xpm"
-#include "pixmaps/cssspan.xpm"
-#include "pixmaps/cssdiv.xpm"
-#include "pixmaps/cssclass.xpm"
-#include "pixmaps/cssstyle.xpm"
-#include "pixmaps/linkjscript.xpm"
-#include "pixmaps/jscript.xpm"
-#include "pixmaps/linkstylesheet.xpm"
-#include "pixmaps/wml_a.xpm"
-#include "pixmaps/wml_access.xpm"
-#include "pixmaps/wml_bold.xpm"
-#include "pixmaps/wml_break.xpm"
-#include "pixmaps/wml_card.xpm"
-#include "pixmaps/wml_do.xpm"
-#include "pixmaps/wml_go.xpm"
-#include "pixmaps/wml_italic.xpm"
-#include "pixmaps/wml_nbsp.xpm"
-#include "pixmaps/wml_paragraph.xpm"
-#include "pixmaps/wml_standard_document.xpm"
-#include "pixmaps/wml_timer.xpm"
-#include "pixmaps/wml_postfield.xpm"
-#include "pixmaps/wml_var.xpm"
-#include "pixmaps/wml_prev.xpm"
-#include "pixmaps/wml_refresh.xpm"
-#include "pixmaps/wml_noop.xpm"
+#include "../pixmaps/new.xpm"
+#include "../pixmaps/close.xpm"
+#include "../pixmaps/copy.xpm"
+#include "../pixmaps/cut.xpm"
+#include "../pixmaps/help.xpm"
+#include "../pixmaps/yes.xpm"
+#include "../pixmaps/no.xpm"
+#include "../pixmaps/ok.xpm"
+#include "../pixmaps/cancel.xpm"
+#include "../pixmaps/open.xpm"
+#include "../pixmaps/open_small.xpm"
+#include "../pixmaps/paste.xpm"
+#include "../pixmaps/preferences.xpm"
+#include "../pixmaps/print.xpm"
+#include "../pixmaps/redo.xpm"
+#include "../pixmaps/save.xpm"
+#include "../pixmaps/save_as.xpm"
+#include "../pixmaps/search-replace.xpm"
+#include "../pixmaps/search.xpm"
+#include "../pixmaps/spellcheck.xpm"
+#include "../pixmaps/undo.xpm"
+#include "../pixmaps/quick_start.xpm"
+#include "../pixmaps/body.xpm"
+#include "../pixmaps/bold.xpm"
+#include "../pixmaps/italic.xpm"
+#include "../pixmaps/underline.xpm"
+#include "../pixmaps/strikeout.xpm"
+#include "../pixmaps/paragraph.xpm"
+#include "../pixmaps/break.xpm"
+#include "../pixmaps/nbsp.xpm"
+#include "../pixmaps/anchor.xpm"
+#include "../pixmaps/image.xpm"
+#include "../pixmaps/thumbnail.xpm"
+#include "../pixmaps/hrule.xpm"
+#include "../pixmaps/center.xpm"
+#include "../pixmaps/right.xpm"
+#include "../pixmaps/comment.xpm"
+#include "../pixmaps/email.xpm"
+#include "../pixmaps/font.xpm"
+#include "../pixmaps/font_base.xpm"
+#include "../pixmaps/fontp1.xpm"
+#include "../pixmaps/fontm1.xpm"
+#include "../pixmaps/fontpre.xpm"
+#include "../pixmaps/fontsub.xpm"
+#include "../pixmaps/fontsuper.xpm"
+#include "../pixmaps/headings.xpm"
+#include "../pixmaps/fonth1.xpm"
+#include "../pixmaps/fonth2.xpm"
+#include "../pixmaps/fonth3.xpm"
+#include "../pixmaps/fonth4.xpm"
+#include "../pixmaps/fonth5.xpm"
+#include "../pixmaps/fonth6.xpm"
+#include "../pixmaps/tablewhiz.xpm"
+#include "../pixmaps/table.xpm"
+#include "../pixmaps/table-tr.xpm"
+#include "../pixmaps/table-th.xpm"
+#include "../pixmaps/table-td.xpm"
+#include "../pixmaps/table2.xpm"
+#include "../pixmaps/table-tr2.xpm"
+#include "../pixmaps/table-th2.xpm"
+#include "../pixmaps/table-td2.xpm"
+#include "../pixmaps/table-capt.xpm"
+#include "../pixmaps/framewhiz.xpm"
+#include "../pixmaps/frameset.xpm"
+#include "../pixmaps/frame.xpm"
+#include "../pixmaps/frameset2.xpm"
+#include "../pixmaps/frame2.xpm"
+#include "../pixmaps/frame_no.xpm"
+#include "../pixmaps/frame_base.xpm"
+#include "../pixmaps/form.xpm"
+#include "../pixmaps/form_submit.xpm"
+#include "../pixmaps/form_reset.xpm"
+#include "../pixmaps/form_text.xpm"
+#include "../pixmaps/form_hidden.xpm"
+#include "../pixmaps/form_textarea.xpm"
+#include "../pixmaps/form_radio.xpm"
+#include "../pixmaps/form_check.xpm"
+#include "../pixmaps/form_select.xpm"
+#include "../pixmaps/form_option.xpm"
+#include "../pixmaps/list.xpm"
+#include "../pixmaps/list_ul.xpm"
+#include "../pixmaps/list_ol.xpm"
+#include "../pixmaps/list_li.xpm"
+#include "../pixmaps/list_dl.xpm"
+#include "../pixmaps/list_dt.xpm"
+#include "../pixmaps/list_dd.xpm"
+#include "../pixmaps/list_menu.xpm"
+#include "../pixmaps/image_map.xpm"
+#include "../pixmaps/meta.xpm"
+#include "../pixmaps/embed.xpm"
+#include "../pixmaps/netscape.xpm"
+#include "../pixmaps/color_select.xpm"
+#include "../pixmaps/color_select_small.xpm"
+#include "../pixmaps/time.xpm"
+#include "../pixmaps/nothing.xpm"
+#include "../pixmaps/php3.xpm"
+#include "../pixmaps/ssi.xpm"
+#include "../pixmaps/rxml.xpm"
+#include "../pixmaps/breakall.xpm"
+/* #include "../pixmaps/dtd.xpm"
+#include "../pixmaps/head.xpm" */
+#include "../pixmaps/cssnewstyle.xpm"
+#include "../pixmaps/cssspan.xpm"
+#include "../pixmaps/cssdiv.xpm"
+#include "../pixmaps/cssclass.xpm"
+#include "../pixmaps/cssstyle.xpm"
+#include "../pixmaps/css_small.xpm"
+#include "../pixmaps/linkjscript.xpm"
+#include "../pixmaps/jscript.xpm"
+#include "../pixmaps/linkstylesheet.xpm"
+#include "../pixmaps/bluefish_small.xpm"
+#include "../pixmaps/wml_a.xpm"
+#include "../pixmaps/wml_access.xpm"
+#include "../pixmaps/wml_bold.xpm"
+#include "../pixmaps/wml_break.xpm"
+#include "../pixmaps/wml_card.xpm"
+#include "../pixmaps/wml_do.xpm"
+#include "../pixmaps/wml_go.xpm"
+#include "../pixmaps/wml_italic.xpm"
+#include "../pixmaps/wml_nbsp.xpm"
+#include "../pixmaps/wml_paragraph.xpm"
+#include "../pixmaps/wml_standard_document.xpm"
+#include "../pixmaps/wml_timer.xpm"
+#include "../pixmaps/wml_postfield.xpm"
+#include "../pixmaps/wml_var.xpm"
+#include "../pixmaps/wml_prev.xpm"
+#include "../pixmaps/wml_refresh.xpm"
+#include "../pixmaps/wml_noop.xpm"
+#include "../pixmaps/abbr.xpm"
+#include "../pixmaps/acronym.xpm"
+#include "../pixmaps/cite.xpm"
+#include "../pixmaps/code.xpm"
+#include "../pixmaps/dfn.xpm"
+#include "../pixmaps/kbd.xpm"
+#include "../pixmaps/samp.xpm"
+#include "../pixmaps/var.xpm"
+#include "../pixmaps/javascript_mouseover.xpm"
+#include "../pixmaps/javascript_netscape.xpm"
 
 
 GtkWidget *new_pixmap(int type, GdkWindow * window, GdkColor * background)
@@ -372,6 +386,10 @@ GtkWidget *new_pixmap(int type, GdkWindow * window, GdkColor * background)
 	case 158:
 		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, form_select_xpm);
 		break;
+
+	case 417:
+		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, form_option_xpm);
+		break;
 	case 159:
 		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, list_xpm);
 		break;
@@ -445,12 +463,14 @@ GtkWidget *new_pixmap(int type, GdkWindow * window, GdkColor * background)
 	case 184:
 		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, cssstyle_xpm);
 		break;
+/*
 	case 185:
 		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, dtd_xpm);
 		break;
 	case 186:
 		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, head_xpm);
 		break;
+	*/
 	case 187:
 		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, linkjscript_xpm);
 		break;
@@ -460,6 +480,15 @@ GtkWidget *new_pixmap(int type, GdkWindow * window, GdkColor * background)
 	case 189:
 		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, linkstylesheet_xpm);
 		break;
+	case 190:
+		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, bluefish_small_xpm);
+		break;
+	case 191:
+		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, headings_xpm);
+		break;
+	case 192:
+		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, css_small_xpm);
+		break;
 
 		/* everything else */
 	case 200:
@@ -535,9 +564,43 @@ GtkWidget *new_pixmap(int type, GdkWindow * window, GdkColor * background)
 	case 416:
 		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, wml_access_xpm);
 		break;
+	/* html formatting by context pixmaps */
+	case 418: /* acronym */
+		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, acronym_xpm);
+		break;
+	case 419: /* cite */
+		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, cite_xpm);
+		break;
+	case 420: /* code */
+		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, code_xpm);
+		break;
+	case 421: /* dfn */
+		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, dfn_xpm);
+		break;
+	case 422: /* kbd */
+		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, kbd_xpm);
+		break;
+	case 423: /* samp */
+		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, samp_xpm);
+		break;
+	case 424: /* var */
+		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, var_xpm);
+		break;
+	case 425: /* abbr */
+		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, abbr_xpm);
+		break;		
+		/* javascript pixmaps */
+	case 500:
+		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, javascript_mouseover_xpm);
+		break;
+	case 501:
+		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, javascript_netscape_xpm);
+		break;
+
 	}
 
 	wpixmap = gtk_pixmap_new(pixm, mask);
 	gdk_pixmap_unref(pixm);
+	gdk_bitmap_unref(mask);
 	return wpixmap;
 }
