@@ -20,14 +20,17 @@
 #include "default_include.h"
 
 #include <locale.h>
-
+#include <string.h>  /* memset() */
 
 #include "bluefish.h"
+#include "bf_lib.h"
 #include "stringlist.h"			/*add to stringlist */
 #include "menu.h"				/* menu translate */
 #include "gtk_easy.h"			/* window_with_title */
 #include "init.h"				/* make_config_list_item */
-#include "callbacks.h"			/* file_close_cb */
+#include "document.h"			/* file_close_cb */
+#include "project.h"			/* myself ;-) */
+#include "interface.h"  /* notebook_changed() */
 
 /* this struct is used for the project editor */
 typedef struct {
@@ -47,41 +50,33 @@ static void project_edit_list_lcb(GtkWidget * widget, gpointer data)
 	switch (GPOINTER_TO_INT(data)) {
 	case 1:
 		DEBUG_MSG("project_edit_list, before, main_v->current_project.filenames(%p)\n", main_v->current_project.filenames);
-		main_v->current_project.filenames =
-			edit_stringlist(main_v->current_project.filenames, _("Bluefish project - files"), GPOINTER_TO_INT(data));
+		estrl_dialog(&main_v->current_project.filenames, _("Bluefish project - files"), GPOINTER_TO_INT(data), 0, NULL, NULL);
 		DEBUG_MSG("project_edit_list, after, main_v->current_project.filenames(%p)\n", main_v->current_project.filenames);
 		break;
 	case 2:
-		main_v->current_project.colorlist =
-			edit_stringlist(main_v->current_project.colorlist, _("Bluefish project - colorlist"), GPOINTER_TO_INT(data));
+		estrl_dialog(&main_v->current_project.colorlist, _("Bluefish project - colorlist"), GPOINTER_TO_INT(data), 0, NULL, NULL);
 		break;
 	case 3:
-		main_v->current_project.fontlist =
-			edit_stringlist(main_v->current_project.fontlist, _("Bluefish project - fontlist"), GPOINTER_TO_INT(data));
+		estrl_dialog(&main_v->current_project.fontlist, _("Bluefish project - fontlist"), GPOINTER_TO_INT(data), 0, NULL, NULL);
 		break;
 	case 4:
-		main_v->current_project.urllist =
-			edit_stringlist(main_v->current_project.urllist, _("Bluefish project - urllist"), GPOINTER_TO_INT(data));
+		estrl_dialog(&main_v->current_project.urllist, _("Bluefish project - urllist"), GPOINTER_TO_INT(data), 0, NULL, NULL);
 		break;
 	case 5:
 		main_v->current_project.targetlist = add_to_stringlist(main_v->current_project.targetlist, "_top");
 		main_v->current_project.targetlist = add_to_stringlist(main_v->current_project.targetlist, "_blank");
 		main_v->current_project.targetlist = add_to_stringlist(main_v->current_project.targetlist, "_self");
 		main_v->current_project.targetlist = add_to_stringlist(main_v->current_project.targetlist, "_parent");
-		main_v->current_project.targetlist =
-			edit_stringlist(main_v->current_project.targetlist, _("Bluefish project - targetlist"), GPOINTER_TO_INT(data));
+		estrl_dialog(&main_v->current_project.targetlist, _("Bluefish project - targetlist"), GPOINTER_TO_INT(data), 0, NULL, NULL);
 		break;
 	case 6:
-		main_v->current_project.dtd_cblist =
-			edit_stringlist(main_v->current_project.dtd_cblist, _("Bluefish project - dtd_cblist"), GPOINTER_TO_INT(data));
+		estrl_dialog(&main_v->current_project.dtd_cblist, _("Bluefish project - dtd_cblist"), GPOINTER_TO_INT(data), 0, NULL, NULL);
 		break;
 	case 7:
-		main_v->current_project.classlist =
-			edit_stringlist(main_v->current_project.classlist, _("Bluefish project - classlist"), GPOINTER_TO_INT(data));
+		estrl_dialog(&main_v->current_project.classlist, _("Bluefish project - classlist"), GPOINTER_TO_INT(data), 0, NULL, NULL);
 		break;
 	case 8:
-		main_v->current_project.metalist =
-			edit_stringlist(main_v->current_project.metalist, _("Bluefish project - metalist"), GPOINTER_TO_INT(data));
+		estrl_dialog(&main_v->current_project.headerlist, _("Bluefish project - headers"), GPOINTER_TO_INT(data), 0, NULL, NULL);
 		break;
 	default:
 
@@ -89,12 +84,13 @@ static void project_edit_list_lcb(GtkWidget * widget, gpointer data)
 	}
 }
 
-static void project_edit_cancel_lcb(GtkWidget * widget, Teditproject * data)
-{
-	DEBUG_MSG("project_edit_cancel, started\n");
+static void project_edit_destroy_lcb(GtkWidget * widget, GdkEvent *event, Teditproject * data) {
 	window_destroy(data->window);
 	g_free(data);
-	DEBUG_MSG("project_edit_cancel, finished\n");
+}
+
+static void project_edit_cancel_lcb(GtkWidget * widget, gpointer data) {
+	project_edit_destroy_lcb(NULL, NULL, data);
 }
 
 
@@ -105,7 +101,7 @@ static void project_edit_ok_lcb(GtkWidget * widget, Teditproject * data)
 	string_apply(&main_v->current_project.template, data->templ_entry);
 	DEBUG_MSG("project_edit_ok, main_v->current_project.webdir(%p)=%s\n", main_v->current_project.webdir, main_v->current_project.webdir);
 	main_v->current_project.modified = 1;
-  project_edit_cancel_lcb(widget, data);
+	project_edit_destroy_lcb(NULL, NULL, data);
 }
 
 
@@ -148,7 +144,7 @@ void project_add_document_cb(GtkWidget * widget, gpointer data)
  *****************************************************/
 
 
-GtkWidget *string_entry(gchar * labeltext, GtkWidget * table, gchar * which_config_string, gint left, gint right, gint top, gint bottom)
+static GtkWidget *string_entry(gchar * labeltext, GtkWidget * table, gchar * which_config_string, gint left, gint right, gint top, gint bottom)
 {
 
 	GtkWidget *label, *tmpentry;
@@ -171,10 +167,10 @@ static GtkItemFactoryEntry menu_items[] = {
 	{N_("/Lists/Targets"), NULL, project_edit_list_lcb, 5, NULL},
 	{N_("/Lists/Document type definitions"), NULL, project_edit_list_lcb, 6, NULL},
 	{N_("/Lists/CSS classes"), NULL, project_edit_list_lcb, 7, NULL},
-	{N_("/Lists/META tags"), NULL, project_edit_list_lcb, 8, NULL}
+	{N_("/Lists/headers"), NULL, project_edit_list_lcb, 8, NULL}
 };
 
-void get_project_edit_menu(GtkWidget * window, GtkWidget ** menubar)
+static void get_project_edit_menu(GtkWidget * window, GtkWidget ** menubar)
 {
 	GtkItemFactory *item_factory;
 	GtkAccelGroup *accel_group;
@@ -196,7 +192,7 @@ void project_edit_cb(GtkWidget * widget, gpointer data)
 
 	editproject = g_malloc(sizeof(Teditproject));
 	editproject->window = window_full(main_v->current_project.filename, GTK_WIN_POS_MOUSE
-		, GTK_WINDOW_TOPLEVEL, 0, GTK_SIGNAL_FUNC(project_edit_cancel_lcb), editproject);
+		, GTK_WINDOW_TOPLEVEL, 5, project_edit_destroy_lcb, editproject);
 
 	table = gtk_table_new(8, 4, 1);
 	gtk_container_add(GTK_CONTAINER(editproject->window), table);
@@ -227,7 +223,7 @@ void project_edit_cb(GtkWidget * widget, gpointer data)
  *        other project functions
  *
  ********************************************/
-GList *create_project_list(GList * project_list)
+static GList *create_project_list(GList * project_list)
 {
 	if (project_list == NULL) {
 		/* this should be done only once !!!!!!!!!!! */
@@ -240,44 +236,100 @@ GList *create_project_list(GList * project_list)
 		project_list = make_config_list_item(project_list, &main_v->current_project.colorlist, 'l', "colorlist:");
 		project_list = make_config_list_item(project_list, &main_v->current_project.dtd_cblist, 'l', "dtd_cblist:");
 		project_list = make_config_list_item(project_list, &main_v->current_project.classlist, 'l', "classlist:");
-		project_list = make_config_list_item(project_list, &main_v->current_project.metalist, 'l', "metalist:");
+		project_list = make_config_list_item(project_list, &main_v->current_project.headerlist, 'l', "metalist:");
 		project_list = make_config_list_item(project_list, &main_v->current_project.filenames, 'l', "filenames:");
 	}
 	return project_list;
 }
 
-
-void project_empty(void)
-{
+static gint test_modified_docs_in_project() {
 
 	GList *tmpdoclist, *tmpprolist;
 
+	DEBUG_MSG("test_modified_docs_in_project, doclist len=%d, prolist len=%d\n"
+			,g_list_length(main_v->documentlist)
+			,g_list_length(main_v->current_project.filenames));
+	tmpdoclist = g_list_first(main_v->documentlist);
+	while (tmpdoclist) {
+		tmpprolist = g_list_first(main_v->current_project.filenames);
+		while (tmpprolist) {
+			if (((Tdocument *)tmpdoclist->data)->filename && (strcmp(((Tdocument *)tmpdoclist->data)->filename, (gchar *) tmpprolist->data) == 0)) {
+				if (((Tdocument *)tmpdoclist->data)->modified) {
+					return 1;
+				}
+			}
+			tmpprolist = g_list_next(tmpprolist);
+		}
+		tmpdoclist = g_list_next(tmpdoclist);
+	}
+	return 0;
+}
+
+
+static void project_empty(void)
+{
+
+	GList *tmpdoclist, *tmpprolist;
+	Tdocument *tmpdoc;
+	gint retval;
+
 	DEBUG_MSG("project_close, closing files that are in current project\n");
 	DEBUG_MSG("project_close, project filenames(%p)\n", main_v->current_project.filenames);
+
+	if (test_modified_docs_in_project()) {
+		gchar *options[] = {N_("save all"), N_("close all"), N_("choose per file"), N_("cancel"), NULL};
+		retval =	multi_button_dialog(_("Bluefish warning: some file(s) are modified!"), 3, _("Some file(s) in your project are modified\nplease choose your action"), options);
+		if (retval == 3) {
+			DEBUG_MSG("project_empty, cancel clicked, returning 0\n");
+			return;
+		}
+	} else {
+		retval = 1;
+	}
+
 	tmpdoclist = g_list_first(main_v->documentlist);
-	while (tmpdoclist != NULL) {
-		if (tmpdoclist->data != NULL) {
-			main_v->current_document = (Tdocument *) tmpdoclist->data;
-			tmpprolist = g_list_first(main_v->current_project.filenames);
-			while (tmpprolist != NULL) {
-				if ((main_v->current_document->filename != NULL) && (tmpprolist->data != NULL)) {
-					DEBUG_MSG("project_close, filename(%s) data(%s)\n", main_v->current_document->filename, (char *) tmpprolist->data);
-					if (strcmp(main_v->current_document->filename, (gchar *) tmpprolist->data) == 0) {
-						DEBUG_MSG("project_close, filename %s can be closed\n", main_v->current_document->filename);
-						tmpdoclist = g_list_previous(tmpdoclist);
-						file_close_cb(NULL, NULL);
-						tmpprolist = g_list_last(tmpprolist);
+	while (tmpdoclist) {
+		tmpdoc = (Tdocument *) tmpdoclist->data;
+		if (test_only_empty_doc_left(tmpdoc)) {
+			DEBUG_MSG("project_empty, only an empty doc left, break loop\n");
+			break;
+		}
+	
+		tmpprolist = g_list_first(main_v->current_project.filenames);
+		while (tmpprolist && tmpdoclist) {
+			if (strcmp(tmpdoc->filename, (gchar *) tmpprolist->data) == 0) {
+				switch (retval) {
+				case 0:
+					doc_save(tmpdoc, 0, 0);
+					doc_destroy(tmpdoc);
+					tmpdoclist = NULL;
+				break;
+				case 1:
+					doc_destroy(tmpdoc);
+					tmpdoclist = NULL;
+				break;
+				case 2:
+					if (doc_close(tmpdoc, 0)) {
+						tmpdoclist = NULL;
+					} else {
+						notebook_changed();
+						return;
 					}
+				break;
+				default:
+					notebook_changed();
+					return;
+				break;
 				}
+				tmpprolist = g_list_last(tmpprolist);
+			} else {
 				tmpprolist = g_list_next(tmpprolist);
 			}
 		}
-		if (tmpdoclist) {
-			tmpdoclist = g_list_next(tmpdoclist);
+		if (tmpdoclist == NULL) {
+			tmpdoclist = g_list_first(main_v->documentlist);
 		} else {
-			if (g_list_length(main_v->documentlist) > 0) {
-				tmpdoclist = g_list_first(main_v->documentlist);
-			}
+			tmpdoclist = g_list_next(tmpdoclist);
 		}
 	}
 
@@ -291,16 +343,20 @@ void project_empty(void)
 	free_stringlist(main_v->current_project.colorlist);
 	free_stringlist(main_v->current_project.dtd_cblist);
 	free_stringlist(main_v->current_project.classlist);
-	free_stringlist(main_v->current_project.metalist);
+	free_stringlist(main_v->current_project.headerlist);
+	DEBUG_MSG("empty_project, doclist len=%d, prolist len=%d\n"
+			,g_list_length(main_v->documentlist)
+			,g_list_length(main_v->current_project.filenames));
 	free_stringlist(main_v->current_project.filenames);
-
 	memset(&main_v->current_project, 0, sizeof(Tproject));
+	DEBUG_MSG("empty_project, doclist len=%d, prolist len=%d\n"
+			,g_list_length(main_v->documentlist)
+			,g_list_length(main_v->current_project.filenames));	
 	main_v->current_project.modified = 0;
 }
 
 void project_from_file(void)
 {
-	GList *tmplist;
 	gchar *tmpstring;
 
 	DEBUG_MSG("project_from_file, started, checking if project_list exists\n");
@@ -322,16 +378,14 @@ void project_from_file(void)
 	gtk_window_set_title(GTK_WINDOW(main_v->main_window), tmpstring);
 	DEBUG_MSG("project_from_file, setting title to %s\n", tmpstring);
 	g_free(tmpstring);
+
 	/* then we should open the files from the project */
-	tmplist = g_list_first(main_v->current_project.filenames);
-	while (tmplist != NULL) {
-		DEBUG_MSG("project_from_file, opening files in project, tmplist=%p, tmplist->data=%s\n", tmplist, (char *) tmplist->data);
-		file_open((gchar *) tmplist->data);
-		tmplist = g_list_next(tmplist);
-	}
-	DEBUG_MSG("project_from_file, opening files in project finished\n");
+	docs_new_from_files(main_v->current_project.filenames);
+
 	main_v->current_project.modified = 0;
 	change_dir(main_v->current_project.basedir);
+  populate_dir_file_list();
+	populate_dir_history();
 }
 
 void project_to_file(void)
@@ -383,6 +437,9 @@ void project_open_cb(GtkWidget * widget, gpointer data)
 
 void project_close_cb(GtkWidget * widget, gpointer data)
 {
+	if (main_v->current_project.filename != NULL) {
+		project_to_file();
+	}
 	project_empty();
 }
 
