@@ -17,7 +17,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-
 /*
  * THIS IS THE BACKEND FLOW DIAGRAM
  *
@@ -47,18 +46,38 @@
 
 
 #include "bluefish.h"
+#include "bf_lib.h"
 #include "document.h"			/* doc_replace_text() */
-#include "interface.h"			/* switch_to_document */
+#include "interface.h"			/* g */
 #include "gtk_easy.h"         /* a lot of GUI functions */
 #include "snr2.h"
 
 typedef enum { string, upcase, lowcase }replacetypes;
 
 typedef struct {
+	/* span of characters */
 	gint start;
 	gint end;
+
+	/* span of byte-data in buffer */
+	gint bstart;
+	gint bend;
 } Tsearch_result;
 
+/* Note:
+    If Gtk's multibyte text support was enabled, a byte-offset
+  of a string can not be used as a character-offset of GtkText.
+    The member "start" and "end" are character-offset.
+    The member "bstart" and "bend" are byte-offset in the source
+  buffer.
+    These values are set in search_backend().
+    In single-byte text mode,
+        start == bstart, end == bend .
+
+   iwasa
+*/
+
+
 typedef struct {
 	gint start;
 	gint end;
@@ -126,7 +145,32 @@ static void reset_last_snr2(void) {
 
 /***********************************************************/
 
-Tsearch_result search_backend(gchar *pattern, gint is_regex, gint is_case_sens, gchar *buf) {
+/* for multibyte-text support.
+   wchar_len() returns -1 if it failed to convert.
+*/
+static
+gint wchar_len(gchar *src, gint src_len)
+{
+	gchar *tmpbuf;
+	GdkWChar *tmpwbuf;
+	gint convlen = -1;
+
+	if (src_len == 0)
+		return 0;
+	tmpbuf = g_strndup(src, src_len);
+	if (tmpbuf) {
+		tmpwbuf = g_new(GdkWChar, src_len);
+		if (tmpwbuf) {
+			convlen = gdk_mbstowcs(tmpwbuf, tmpbuf, src_len);
+			g_free(tmpwbuf);
+		}
+		g_free(tmpbuf);
+	}
+	return convlen;
+}
+
+
+Tsearch_result search_backend(gchar *pattern, gint is_regex, gint is_case_sens, gchar *buf, gboolean use_mb) {
 
 	Tsearch_result returnvalue;
 	int (*f) ();
@@ -136,6 +180,8 @@ Tsearch_result search_backend(gchar *pattern, gint is_regex, gint is_case_sens,
 
 	returnvalue.start = -1;
 	returnvalue.end = -1;
+	returnvalue.bstart = -1;
+	returnvalue.bend = -1;
 	if ((!pattern) || (!buf)) {
 		DEBUG_MSG("search_backend, pattern or buf is NULL\n");
 		return returnvalue;
@@ -152,10 +198,9 @@ Tsearch_result search_backend(gchar *pattern, gint is_regex, gint is_case_sens,
 			pmatch[0].rm_so = -1;
 			pmatch[0].rm_eo = -1;
 		} 
-		returnvalue.start = pmatch[0].rm_so;
-		returnvalue.end = pmatch[0].rm_eo;
+		returnvalue.bstart = pmatch[0].rm_so;
+		returnvalue.bend = pmatch[0].rm_eo;
 		regfree(&reg_pat);
-		return returnvalue;
 		/* regex part end */
 	} else {
 		/* non regex part start */
@@ -170,13 +215,31 @@ Tsearch_result search_backend(gchar *pattern, gint is_regex, gint is_case_sens,
 		for (i = 0; i <= (buflen - patlen); i++) {
 			match = f(&buf[i], pattern, patlen);
 			if (match == 0) {
-				returnvalue.start = i;
-				returnvalue.end = i + patlen;
-				return returnvalue;
+				returnvalue.bstart = i;
+				returnvalue.bend = i + patlen;
+				break;
 			}
 		}
 		/* non regex part end */	
 	}
+
+	if (use_mb) {
+		if (returnvalue.bstart >= 0) {
+			returnvalue.start = wchar_len(buf, returnvalue.bstart);
+		}
+		if (returnvalue.bend >= 0) {
+			returnvalue.end = wchar_len(buf, returnvalue.bend);
+		}
+		if (returnvalue.start < 0 || returnvalue.end < 0) {
+			returnvalue.start = -1;
+			returnvalue.end = -1;
+			returnvalue.bstart = -1;
+			returnvalue.bend = -1;
+		}
+	} else {
+		returnvalue.start = returnvalue.bstart;
+		returnvalue.end = returnvalue.bend;
+	}
 	return returnvalue;
 }
 
@@ -188,11 +251,17 @@ Tsearch_result search_doc(Tdocument *document, gchar *pattern, gint is_regex, gi
 
 	DEBUG_MSG("search_doc, started\n");
 	fulltext = gtk_editable_get_chars(GTK_EDITABLE(document->textbox), startpos, -1);
-	result = search_backend(pattern, is_regex, is_case_sens, fulltext);
+	result = search_backend(pattern, is_regex, is_case_sens, fulltext, GTK_TEXT(document->textbox)->use_wchar);
 	g_free(fulltext);
-	last_snr2.result.start = result.start + startpos;
-	last_snr2.result.end = result.end + startpos;
-	last_snr2.doc = document;
+	if (result.end > 0) {
+		last_snr2.result.start = result.start + startpos;
+		last_snr2.result.end = result.end + startpos;
+		last_snr2.doc = document;
+	} else {
+		last_snr2.result.start = -1;
+		last_snr2.result.end =  -1;
+		last_snr2.doc = document;
+	}
 	DEBUG_MSG("search_doc, result.start=%d, result.end=%d\n", result.start, result.end);
 	return result;
 }
@@ -204,10 +273,10 @@ void doc_show_result(Tdocument *document, gint start, gint end) {
 	if (end > 0) {
 		DEBUG_MSG("search_show_doc, start=%d, end=%d\n",start, end);
 		if (document != main_v->current_document) {
-			switch_to_document(document);
+			switch_to_document_by_pointer(document);
 		}
 		linenum = position_to_linenum(end);
-		doc_go_to_line(linenum + 1, 0);
+		go_to_line(linenum + 1, 0);
 		gtk_editable_select_region(GTK_EDITABLE(document->textbox), start, end);
 	}
 }
@@ -246,14 +315,14 @@ Tsearch_all_result search_all(gchar *pattern, gint is_regex, gint is_case_sens)
 
 /*****************************************************/
 
-Tsearch_result replace_backend(gchar *pattern, gint is_regex, gint is_case_sens, gchar *buf, gchar *replace_string, Tdocument *doc, gint offset, replacetypes replacetype) {
+Tsearch_result replace_backend(gchar *pattern, gint is_regex, gint is_case_sens, gchar *buf, gchar *replace_string, Tdocument *doc, gint offset, replacetypes replacetype, gint use_mb) {
 /* the offset in this function is the difference between the buffer and the text widget because of previous replace 
 actions, so the first char in buf is actually number offset in the text widget */
 
 	Tsearch_result result;
 	gchar *tmpstr=NULL;
 	
-	result = search_backend(pattern, is_regex, is_case_sens, buf);
+	result = search_backend(pattern, is_regex, is_case_sens, buf, use_mb);
 	DEBUG_MSG("replace_backend, offset=%d, result.start=%d, result.end=%d\n", offset, result.start, result.end);
 	if (result.end > 0) {
 		switch (replacetype) {
@@ -261,11 +330,11 @@ actions, so the first char in buf is actually number offset in the text widget *
 			tmpstr = g_strdup(replace_string);
 		break;
 		case upcase:
-			tmpstr = g_strndup(&buf[result.start], result.end - result.start);
+			tmpstr = g_strndup(&buf[result.bstart], result.bend - result.bstart);
 			g_strup(tmpstr);
 		break;
 		case lowcase:
-			tmpstr = g_strndup(&buf[result.start], result.end - result.start);
+			tmpstr = g_strndup(&buf[result.bstart], result.bend - result.bstart);
 			g_strdown(tmpstr);
 		break;
 		}
@@ -284,10 +353,16 @@ Tsearch_result replace_doc_once(gchar *pattern, gint is_regex, gint is_case_sens
 	Tsearch_result result;
 
 	fulltext = gtk_editable_get_chars(GTK_EDITABLE(doc->textbox), startpos, endpos);
-	result = replace_backend(pattern, is_regex, is_case_sens, fulltext, replace_string, doc, startpos, replacetype);
-	last_snr2.result.start = result.start + startpos;
-	last_snr2.result.end = result.end + startpos;
-	last_snr2.doc = doc;
+	result = replace_backend(pattern, is_regex, is_case_sens, fulltext, replace_string, doc, startpos, replacetype, GTK_TEXT(doc->textbox)->use_wchar);
+	if ( result.end > 0) {
+		last_snr2.result.start = result.start + startpos;
+		last_snr2.result.end = result.end + startpos;
+		last_snr2.doc = doc;
+	} else {
+		last_snr2.result.start = -1;
+		last_snr2.result.end = -1;
+		last_snr2.doc = doc;
+	}
 	g_free(fulltext);
 	return result;
 }
@@ -297,39 +372,48 @@ Tsearch_result replace_doc_once(gchar *pattern, gint is_regex, gint is_case_sens
 void replace_doc_multiple(gchar *pattern, gint is_regex, gint is_case_sens, gint startpos, gint endpos, gchar *replace_string, Tdocument *doc, replacetypes replacetype) {
 /* endpos -1 means do till end */
 	gchar *fulltext;
+	gchar *onechar;
 	Tsearch_result result;
 	gint in_buf_offset=0;
 	gint buf_text_offset=startpos;
 	gint fixedreplacelen=0;
 
 	if (replacetype == string) {
-		fixedreplacelen = strlen(replace_string);
+		if (GTK_TEXT(doc->textbox)->use_wchar) {
+			fixedreplacelen = wchar_len(replace_string, strlen(replace_string));
+		} else {
+			fixedreplacelen = strlen(replace_string);
+		}
 	}
 	fulltext = gtk_editable_get_chars(GTK_EDITABLE(doc->textbox), startpos, endpos);
-	result = replace_backend(pattern, is_regex, is_case_sens, fulltext, replace_string, doc, buf_text_offset, replacetype);
-	if (replacetype == string) {
-		buf_text_offset += fixedreplacelen - (result.end - result.start);
-	}
-	if (last_snr2.overlapping_search) {
-		in_buf_offset += result.start + 1;
-	} else {
-		in_buf_offset += result.end;
-	}
-	DEBUG_MSG("replace_doc_multiple, after first search, buf_text_offset=%d, in_buf_offset=%d\n", buf_text_offset, in_buf_offset);
+	result = replace_backend(pattern, is_regex, is_case_sens, fulltext, replace_string, doc, buf_text_offset, replacetype, GTK_TEXT(doc->textbox)->use_wchar);
 
 	while (result.end > 0) {
-		result = replace_backend(pattern, is_regex, is_case_sens, &fulltext[in_buf_offset], replace_string, doc, buf_text_offset + in_buf_offset, replacetype);
-		DEBUG_MSG("replace_doc_multiple, 1- buf_text_offset=%d, in_buf_offset=%d, result.start=%d, result.end=%d\n", buf_text_offset, in_buf_offset, result.start, result.end);
 		if (replacetype == string) {
 			buf_text_offset += fixedreplacelen - (result.end - result.start);
 		}
-
 		if (last_snr2.overlapping_search) {
-			in_buf_offset += result.start + 1;
+			if (GTK_TEXT(doc->textbox)->use_wchar) {
+				onechar = gtk_editable_get_chars(GTK_EDITABLE(doc->textbox), startpos + result.start, startpos + result.start + 1);
+				if (onechar) {
+					in_buf_offset += result.bstart + strlen(onechar);
+				} else {
+					in_buf_offset += result.bstart + 1;
+				}
+				g_free(onechar);
+			} else {
+				in_buf_offset += result.bstart + 1;;
+			}
+			buf_text_offset += result.start + 1;
 		} else {
-			in_buf_offset += result.end;
+			in_buf_offset += result.bend;
+			buf_text_offset += result.end;
 		}
-		DEBUG_MSG("replace_doc_multiple, 2- buf_text_offset=%d, in_buf_offset=%d, result.start=%d, result.end=%d\n", buf_text_offset, in_buf_offset, result.start, result.end);
+		DEBUG_MSG("replace_doc_multiple, after first search, buf_text_offset=%d, in_buf_offset=%d\n", buf_text_offset, in_buf_offset);
+
+		result = replace_backend(pattern, is_regex, is_case_sens, &fulltext[in_buf_offset], replace_string, doc, buf_text_offset, replacetype, GTK_TEXT(doc->textbox)->use_wchar);
+
+		DEBUG_MSG("replace_doc_multiple, 1- buf_text_offset=%d, in_buf_offset=%d, result.start=%d, result.end=%d\n", buf_text_offset, in_buf_offset, result.start, result.end);
 	}
 	last_snr2.result.start = -1;
 	last_snr2.result.end = -1;
@@ -410,7 +494,7 @@ gint replace_prompt_doc(gchar *pattern, gint is_regex, gint is_case_sens, gint s
 	Tsearch_result result;
 
 	fulltext = gtk_editable_get_chars(GTK_EDITABLE(doc->textbox), startpos, endpos);
-	result = search_backend(pattern, is_regex, is_case_sens, fulltext);
+	result = search_backend(pattern, is_regex, is_case_sens, fulltext, GTK_TEXT(doc->textbox)->use_wchar);
 	last_snr2.doc = doc;
 	g_free(fulltext);
 	if (result.end > 0) {
@@ -511,18 +595,19 @@ void snr2_run(void) {
 			doc_show_result(main_v->current_document, result.start + startpos, result.end + startpos);	
 		}
 	}
-
-
-
 }
 
 /*****************************************************/
 
-static void snr2dialog_cancel_lcb(GtkWidget *widget, Tsnr2_win *data) {
-	window_destroy(data->window);
+static void snr2dialog_destroy_lcb(GtkWidget *widget, GdkEvent *event, gpointer data) {
+	window_destroy(((Tsnr2_win *)data)->window);
 	g_free(data);
 }
 
+static void snr2dialog_cancel_lcb(GtkWidget *widget, gpointer data) {
+	snr2dialog_destroy_lcb(NULL, NULL, data);
+}
+
 static void snr2dialog_replacetype_toggled(GtkWidget *widget, Tsnr2_win *data) {
 	gtk_entry_set_editable(GTK_ENTRY(data->replace_string), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data->replacetype_string)));
 }
@@ -556,13 +641,10 @@ static void snr2dialog_ok_lcb(GtkWidget *widget, Tsnr2_win *data) {
 	snr2_run();
 }
 
-
-/*****************************************************/
-
 static void snr2dialog(gint is_replace, gint is_new_search) {
 
 	Tsnr2_win *snr2win;
-	GtkWidget *vbox, *hbox, *vbox2, *vbox3, *frame;
+	GtkWidget *vbox, *hbox, *vbox2, *vbox3, *frame, *but;
 	gchar *tmptext;
 
 	snr2win = g_malloc(sizeof(Tsnr2_win));
@@ -581,8 +663,9 @@ static void snr2dialog(gint is_replace, gint is_new_search) {
 		last_snr2.doc = NULL;
 	}
 	snr2win->window = window_full(tmptext, GTK_WIN_POS_MOUSE, GTK_WINDOW_DIALOG
-						, 5, snr2dialog_cancel_lcb, snr2win);
-	vbox = gtk_vbox_new(FALSE, 0);
+						, 5, snr2dialog_destroy_lcb, snr2win);
+	gtk_window_set_wmclass(GTK_WINDOW(snr2win->window), "Bluefish", "snr");
+	vbox = gtk_vbox_new(FALSE, 1);
 	gtk_container_add(GTK_CONTAINER(snr2win->window), vbox);
 	
 	hbox = gtk_hbox_new(FALSE,0);
@@ -633,10 +716,15 @@ static void snr2dialog(gint is_replace, gint is_new_search) {
 		gtk_signal_connect(GTK_OBJECT(snr2win->replacetype_string), "toggled", 
 				snr2dialog_replacetype_toggled, snr2win);
 	}
-	hbox = gtk_hbox_new(TRUE,3);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 3);
-	gtk_box_pack_start(GTK_BOX(hbox), bf_stock_ok_button(snr2dialog_ok_lcb, snr2win), TRUE, TRUE, 3);
-	gtk_box_pack_start(GTK_BOX(hbox), bf_stock_cancel_button(snr2dialog_cancel_lcb, snr2win), TRUE, TRUE, 3);
+	hbox = gtk_hbutton_box_new();
+	gtk_hbutton_box_set_layout_default(GTK_BUTTONBOX_END);
+	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 1);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+
+	but = bf_stock_ok_button(snr2dialog_ok_lcb, snr2win);
+	gtk_box_pack_start(GTK_BOX(hbox), but, TRUE, TRUE, 0);
+	gtk_window_set_default(GTK_WINDOW(snr2win->window), but);
+	gtk_box_pack_start(GTK_BOX(hbox), bf_stock_cancel_button(snr2dialog_cancel_lcb, snr2win), TRUE, TRUE, 0);
 	gtk_widget_show_all(snr2win->window);
 	
 }
@@ -677,7 +765,7 @@ static gint do_filename_curfile_replace(gchar *fulltext, Tsearch_result result,
 	gchar *replacestring;
 	gint change_lenght = changelen;
 
-	possible_filename = g_strndup(&fulltext[offset + result.start +1], result.end - result.start -2 );
+	possible_filename = g_strndup(&fulltext[offset + result.bstart +1], result.bend - result.bstart -2 );
 	DEBUG_MSG("do_filename_change_replace, possible_filename=%s\n", possible_filename);
 	len = strlen(possible_filename) + strlen(olddirname) + 2;
 	olddirfile = strncat(strncat(strncpy(g_malloc(len), olddirname, len), "/", len), possible_filename, len);
@@ -709,10 +797,12 @@ static gint do_filename_otherfile_replace(gchar *fulltext, Tsearch_result result
 	gint change_length = changelen;
 	gchar *eff_my_filename;
 
-	foundstring = g_strndup(&fulltext[offset + result.start +1], result.end - result.start -2 );
+	foundstring = g_strndup(&fulltext[offset + result.bstart +1], result.bend - result.bstart -2 );
 	possible_filename = most_efficient_filename(create_full_path(foundstring, doc->filename));
 	g_free(foundstring);
-	if (strcmp(possible_filename, newfilename) == 0) {
+	DEBUG_MSG("do_filename_otherfile_replace, posname=%s\n", possible_filename);
+	DEBUG_MSG("do_filename_otherfile_replace, newname=%s\n", oldfilename);
+	if (strcmp(possible_filename, oldfilename) == 0) {
 		eff_my_filename = most_efficient_filename(g_strdup(doc->filename));
 		replacestring = create_relative_link_to(eff_my_filename, newfilename);
 		doc_replace_text(replacestring, result.start + offset + 1 + changelen, result.end + offset + changelen -1, doc);
@@ -745,15 +835,15 @@ void update_filenames_in_file(Tdocument *doc, gchar *oldfilename, gchar *newfile
 	}
 
 	fulltext = gtk_editable_get_chars(GTK_EDITABLE(doc->textbox), 0, -1);
-	result = search_backend(pattern, is_regex, is_case_sens, fulltext);
+	result = search_backend(pattern, is_regex, is_case_sens, fulltext, GTK_TEXT(doc->textbox)->use_wchar);
 	while (result.end > 0) {
 		if (doc_has_newfilename) {
 			changelen = do_filename_curfile_replace(fulltext, result, cur_offset, olddirname, newfilename, changelen, doc);
 		} else {
 			changelen = do_filename_otherfile_replace(fulltext, result, cur_offset, oldfilename, newfilename, changelen, doc);
 		}
-		cur_offset += result.start +1;
-		result = search_backend(pattern, is_regex, is_case_sens, &fulltext[cur_offset]);		
+		cur_offset += result.bstart +1;
+		result = search_backend(pattern, is_regex, is_case_sens, &fulltext[cur_offset], GTK_TEXT(doc->textbox)->use_wchar);
 	}
 	g_free(fulltext);
 	if (doc_has_newfilename) {
