@@ -21,17 +21,38 @@
  * GNU Public Licence).
 */
 
+/* For the select function */
+#define _GNU_SOURCE
+
+
+/*********************************************************************
+ * The spellchecker and it's GUI run like this:
+ *
+ * run_spell_checker
+ * 	create_spc_window
+ * 		spc_start_button_clicked_lcb
+ * 			run_spell_checker
+ * 				create_ispell_pipe
+ * 				--
+ * 				get_next_word
+ * 				check_word
+ * 					ispell_check_word
+ * 					correct_word
+ * 				--
+ * 				ispell_terminate
+ * 				destroy_spc_window
+ * 	
+ **********************************************************************/
+
+
+/* Print debug messages using standard error */
+/* This is needed when the stdin is redirected */
+#define MY_DEBUG_MSG g_printerr
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
-
-#ifdef WITH_SPC
-
-/* for the select function */
-#define _GNU_SOURCE
-
 #include "intl.h"
 #include <glib.h>
 #include <gtk/gtk.h>
@@ -40,6 +61,8 @@
 #include "bluefish.h"
 #include "debug.h"
 #include "gtk_easy.h"
+#include "char_table.h"
+#include "interface.h"
 
 #include <unistd.h>
 #include <fcntl.h>
@@ -66,10 +89,6 @@
 #include <sys/select.h>
 #endif
 
-#define CURRENT_DOCUMENT_TEXT main_v->current_document->textbox
-
-
-
 /* Spellchecker status */
 enum {
 	ISP_OK = 1,
@@ -97,18 +116,30 @@ typedef  struct  {
 	gchar **misses;
 	} isp_result;
 
+/* message from the spell checker GUI */
 gint spc_message;
 
 /*  flag to know if we reach the end of text */
 static gboolean end_of_text;
 
+#define SPC_CHAR_LENGTH 20
+
+/* get_next_char:
+  returns the next char in the text.
+  set flags end_of_text
+  converts special characters like &aacute; to iso characters
+*/
+
 static gchar get_next_char()
 /* FIXME: a more eficient implementation? */
 /* use an HTML parser like gnome-xml ? */
 {
-  gint current_position;
-  gchar c ;
+  gint current_position, old_position,i;
+  gchar c,old_c ;
+  gchar token[SPC_CHAR_LENGTH];
+  DEBUG_MSG("getting next char\n");
   current_position = gtk_text_get_point(GTK_TEXT(CURRENT_DOCUMENT_TEXT));
+  DEBUG_MSG("current_position = %i\n",current_position);
   c = GTK_TEXT_INDEX(GTK_TEXT(CURRENT_DOCUMENT_TEXT) ,current_position);
   /* macro to get a single character */
   if (current_position < gtk_text_get_length(GTK_TEXT(CURRENT_DOCUMENT_TEXT)))
@@ -116,48 +147,126 @@ static gchar get_next_char()
    current_position++;
    gtk_text_set_point(GTK_TEXT(CURRENT_DOCUMENT_TEXT),current_position);
    end_of_text = 0;
+   if (c=='&')
+    {
+     DEBUG_MSG("Character & detected\n");
+     old_position = current_position;
+     token[0] = c;
+     old_c = c;
+     for (i=1;i<SPC_CHAR_LENGTH-2;i++)
+       {
+        c= get_next_char();
+        token[i] = c;
+        if (c==';')
+          break;
+        else if (end_of_text)
+          break;
+       };
+       i++;
+       token[i] = '\0';
+       DEBUG_MSG("Converting special character = \"%s\" \n",token);
+       c = convert_from_html_chars (token,ANY_CHAR_SET);
+       if (c=='\0')
+         {
+           current_position = old_position;
+           c = old_c;
+           /* FIXME: mark for gettext / beter dialog ? */
+           g_printerr("Invalid especial character! %s\n",token);
+         }
+    }
   }
   else
    end_of_text = 1;
   DEBUG_MSG("next_char ='%c'\n",c);
   return (c);
- }
+ } /* end get_next_char */
 
 typedef struct 
 {
  gchar* text;
  gint begin;
  gint end;
+ gint space_length;
+ gint index;
 } Tword;
 
+#define WORD_BUFFER_SIZE 20
+
+inline static Tword* new_word (Tword* word)
+{
+  word=(Tword*) g_malloc(sizeof(Tword));
+  word->index = 0;
+  word->space_length = WORD_BUFFER_SIZE;
+  word->text = g_malloc((word->space_length)*sizeof(gchar));
+  return (word);
+}
+
 inline void free_word (Tword* word)
 {
- DEBUG_MSG("Free word \" %s \" \n",word->text);
+ DEBUG_MSG("Free word \"%s\" \n",word->text);
  g_free(word->text);
  g_free(word);
 }
 
+#define SPECIAL_CHAR_LENGTH 10
+
+static void store_char_in_word (Tword* word,gchar c)
+{
+   DEBUG_MSG("indice =\n %i\n",word->index);
+   /* enlarge the buffer if needed */
+   if (word->index>=word->space_length-1)
+     {
+       DEBUG_MSG("enlarging word buffer\n");
+       word->space_length = word->space_length + WORD_BUFFER_SIZE;
+       DEBUG_MSG("space length=%i\n", word->space_length);
+       word->text = g_realloc (word->text,word->space_length*sizeof(gchar));
+     }
+   (word->text)[word->index] =c;
+   (word->index)++;
+}
+
+inline gboolean is_empty_word(Tword* word)
+{
+  return((word->index)==0);
+}
+
+/* Non alphabetical characters are ignored */
+/* and also the text between < > (asumed to be html tags) */
+/*  in PHP code you can have different numbers of > and <, so this should be extended!
+ like in if (a < b) { */
+/* we want to be sure that only valid characters are pased to ispell */
+
 static Tword* get_next_word(void)
 {
  Tword* next_word;
- gchar c;
+ gchar c ='\0';
  DEBUG_MSG("getting next word\n");
- next_word = (Tword*) g_malloc(sizeof(Tword));
- if (next_word == NULL)
-   {
-    DEBUG_MSG("out of memory in get_next_word!");
-    return next_word;
-   };
+ next_word = new_word(next_word);
  next_word->begin = gtk_text_get_point(GTK_TEXT(CURRENT_DOCUMENT_TEXT));
- while (((c=get_next_char())!=' ') && (!end_of_text) && (c!='<') && (c!='\n'));
+
+ while (!end_of_text)
+  {
+    DEBUG_MSG("Not end of text\n");
+    c=get_next_char();
+    if (isalpha((char)c)||isalpha_iso((char)c))
+     store_char_in_word(next_word,c);
+     /* FIXME: accept iso special characters! */
+    else
+     {
+      DEBUG_MSG("Ignoring non alphabethical character\n");
+      break; /* if it is a non alphabetical character exit the loop */
+     };
+  };
+ store_char_in_word(next_word,'\0');
+ if (is_empty_word(next_word))
+   return  next_word;
  /* at the end of text , do not substract 1 */
  next_word->end = gtk_text_get_point(GTK_TEXT(CURRENT_DOCUMENT_TEXT))-(!end_of_text);
- next_word->text = gtk_editable_get_chars(GTK_EDITABLE(CURRENT_DOCUMENT_TEXT),next_word->begin,next_word->end);
  DEBUG_MSG("next word= \"%s\" \n",next_word-> text);
  /* ignore html tags */
  if (c=='<')
  {
-   DEBUG_MSG("ignoring html tag\n");
+   DEBUG_MSG("Ignoring html tag\n");
    while (((c=get_next_char())!='>') && (!end_of_text));
  }
  return next_word;
@@ -178,22 +287,22 @@ void create_ispell_pipe()
 	struct timeval tv; 
 	int retval = 0;
 
-        DEBUG_MSG ("Creating ispell pipe ...");
+   MY_DEBUG_MSG("Creating ispell pipe ...\n");
         
 	isp_pid = -1;
 
 	if(pipe(pipein)==-1 || pipe(pipeout)==-1) {
-		g_printerr("Bluefish: Can't create pipe for spellchecker!");
+		g_printerr(_("Bluefish: Can't create pipe for spellchecker!"));
 		return;
 	}
 
 	if ((out = fdopen(pipein[1], "w"))==NULL) {
-		g_printerr("Bluefish: Can't create stream for pipe for spellchecker!");
+		g_printerr(_("Bluefish: Can't create stream for pipe for spellchecker!"));
 		return;
 	}
 
 	if ((in = fdopen(pipeout[0], "r"))==NULL) {
-		g_printerr("Bluefish: Can't create stream for pipe for spellchecker!");
+		g_printerr(_("Bluefish: Can't create stream for pipe for spellchecker!"));
 		return;
 	}
 
@@ -204,14 +313,17 @@ void create_ispell_pipe()
 	isp_pid = fork();
 
 	if(isp_pid==-1) {
-		g_printerr("Bluefish: Can't create child process for spellchecker!");
+		g_printerr(_("Bluefish: Can't create child process for spellchecker!"));
 		return;
 	}
 
 	if(isp_pid==0) {        
 		/* child process */
+
+      /* NOTE: Don't use DEBUG_MSG in this part since the output is redirected! */
+
 		
-		DEBUG_MSG ("spell-checker child process \n");
+		MY_DEBUG_MSG ("Spell-checker child process \n");
 		
 		dup2(pipein[0], STDIN_FILENO);
 		dup2(pipeout[1], STDOUT_FILENO);
@@ -224,9 +336,14 @@ void create_ispell_pipe()
 		argv[argc++] = main_v->props.cfg_spc_cline ;
 		argv[argc++] = g_strdup("-a"); // "Pipe" mode
 
-		if (main_v -> props.cfg_spc_lang != "default") {
-			argv[argc++] = g_strdup("-d"); // Dictionary file
-			argv[argc++] = main_v -> props.cfg_spc_lang;
+		if ((g_strcasecmp(main_v -> props.cfg_spc_lang, "default")!=0) &&
+          (*(main_v -> props.cfg_spc_lang)!='\0'))
+      {
+         if (main_v ->props.cfg_spc_lang != NULL)
+         {
+          argv[argc++] = g_strdup("-d"); // Dictionary file
+			 argv[argc++] = main_v -> props.cfg_spc_lang;
+         };
 		}
 
 		if (main_v-> props.spc_accept_compound)
@@ -240,34 +357,43 @@ void create_ispell_pipe()
 		if (main_v->props.spc_use_esc_chars) {
 			/* Specify additional characters that
 			   can be part of a word */
+         if (main_v->props.spc_esc_chars !=NULL) {
 			argv[argc++] = g_strdup("-w"); 
 			/* Put the escape chars in ""s */
 			argv[argc++] = g_strconcat("\"",main_v->props.spc_esc_chars
 			                ,"\"");
+         };
 		}
 		if (main_v->props.spc_use_pers_dict) {
 			/* Specify an alternate personal dictionary */
-			argv[argc++] = g_strdup("-p"); 
-			argv[argc++] = main_v->props.spc_pers_dict;
+          if (main_v->props.spc_pers_dict != NULL)
+          {
+           argv[argc++] = g_strdup("-p");
+			  argv[argc++] = main_v->props.spc_pers_dict;
+          }
+         }
+
+		if (main_v->props.spc_use_input_encoding) {
+         if (main_v->props.spc_input_encoding != NULL) {
+			argv[argc++] = g_strdup("-T"); /* Input encoding */
+			argv[argc++] = main_v->props.spc_input_encoding;
+         }
 		}
-		/*
-		if (spc_use_input_encoding &&
-        	    current_view->currentBuffer()->params.inputenc != "default") {
-			argv[argc++] = g_strdup("-T"); // Input encoding
-			argv[argc++] = current_view->currentBuffer()->params.inputenc.copy();
-		}
-		*/
 
 		argv[argc++] = NULL;
+      #ifdef DEBUG
+      MY_DEBUG_MSG("Executing:\n");
+      for (i=0;i<argc-1;i++)
+       MY_DEBUG_MSG("%s ",argv[i]);
+      #endif
 
 		execvp(main_v->props.cfg_spc_cline , (char * const *) argv);
-		
-		
-		// free the memory used 
+				
+		/* free the memory used  */
 		for (i=0; i < argc -1; i++)
 			g_free(argv[i]);
 		
-		g_printerr("Bluefish: Failed to start ispell!\n");
+		g_printerr(_("Bluefish: Failed to start ispell!\n"));
 		_exit(0);
 	}
 
@@ -299,22 +425,19 @@ void create_ispell_pipe()
 		/* Ok, do the reading. We don't have to FD_ISSET since
 		   there is only one fd in infds. */
 		fgets(buf, 2048, in);
-      DEBUG_MSG ("Ispell pipe created\n");
+      MY_DEBUG_MSG ("\nIspell pipe created\n");
       g_print ("%s\n",buf);
 		
 	} else if (retval == 0) {
 		/* timeout. Give nice message to user. */
 		g_printerr("Ispell read timed out, what now?\n");
-#ifdef WITH_WARNINGS
-#warning Is this the correct thing to do?
-#endif
 		isp_pid = -1;
 		close(pipeout[0]); close(pipeout[1]);
 		close(pipein[0]); close(pipein[1]);
 		isp_fd = -1;
 	} else {
 		/* Select returned error */
-		g_printerr("Select on ispell returned error, what now?\n");
+		g_printerr(_("Select on ispell returned error, what now?\n"));
 	}
 }
 
@@ -341,14 +464,6 @@ inline void ispell_accept_word(char const *word)
 	fputc('\n', out);
 }
 
-static inline void ispell_terminate(void)
-{
-
-	fputs("#\n", out); /* Save personal dictionary */ 
-	fflush(out);
-	fclose(out);
-}
-
 /* Send word to ispell and get reply */
 static isp_result * ispell_check_word (gchar *word)
 {
@@ -462,6 +577,7 @@ static void replace_word_with (Tword *word,gchar* replace_with)
   gtk_editable_insert_text(GTK_EDITABLE(CURRENT_DOCUMENT_TEXT), replace_with,strlen(replace_with),&(word->begin));
   /* DEBUG_MSG("setting new point\n");
   gtk_text_set_point(GTK_TEXT(CURRENT_DOCUMENT_TEXT), word->begin + strlen(replace_with)-1); */
+  /* FIXME: convert iso characters to html ? */
 }
 
 static void correct_word(Tword* word)
@@ -484,10 +600,12 @@ static void correct_word(Tword* word)
 			                  break;
      case SPC_IGNORE:      word_corrected = 1;
                            break;
-     case SPC_REPLACE:     replace_word_with (word,gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(spc_gui.replace_combo)->entry)));
+     case SPC_REPLACE:     replace_word_with (word,gtk_entry_get_text(GTK_ENTRY(spc_gui.replace_entry)));
                            word_corrected = 1;
                            break;
-     /* FIXME: case SPC_STOP */
+     case SPC_CLOSE:       DEBUG_MSG("close message\n");
+                           word_corrected = 1; /* to exit the while loop */
+                           break;
 
      }
  } while (!word_corrected);
@@ -495,71 +613,94 @@ static void correct_word(Tword* word)
  DEBUG_MSG ("Word corrected\n");
 };
 
-
 static void check_word (Tword* word)
 {
  isp_result *result;
+ gchar* converted_string;
  gint i;
- GList* near_misses_list = NULL;
- GList* tmp = NULL;
- spc_message = SPC_NONE;
+
+/* FIXME: Is this really needed ? */
  while (gtk_events_pending())
         gtk_main_iteration();
- DEBUG_MSG ("checking word \"%s\" \n",word->text);
+
  result = ispell_check_word (word->text);
  switch (result->flag) {
    case ISP_MISSED:  /* creates near misses list */
                      DEBUG_MSG("Creating misses list\n");
                      for (i=0; i<result->count; i++)
-                     /* FIXME: use g_list_insert_sorted instead ? */
                      {
+                         if (main_v->props.spc_output_html_chars)
+                           {
+                             converted_string = convert_string_iso_to_html(result->misses[i]);
+                             g_free(result->misses[i]);
+                             result->misses[i] = converted_string;
+                            };
                          DEBUG_MSG("Adding \"%s\" to the near_misses_list\n",result->misses[i]);
-                         near_misses_list = g_list_append(near_misses_list,result->misses[i]);
+                         gtk_clist_append(GTK_CLIST(spc_gui.near_misses_clist),&(result->misses[i]));
                      };
-                     gtk_combo_set_popdown_strings (GTK_COMBO(spc_gui.replace_combo),near_misses_list);
    case ISP_UNKNOWN: gtk_entry_set_text (GTK_ENTRY(spc_gui.word_entry),word->text);
-                     gtk_entry_set_text (GTK_ENTRY(GTK_COMBO(spc_gui.replace_combo)->entry),word->text);
+                     gtk_entry_set_text (GTK_ENTRY(spc_gui.replace_entry),word->text);
                      correct_word(word);
+                     DEBUG_MSG("spc_message = %i\n",spc_message);
    }; /* end switch */
-  if (result->flag ==ISP_MISSED)
-                                {
-                                  DEBUG_MSG("free near_misses_lists ...\n");
-                                  tmp = near_misses_list;
-                                  while (tmp != NULL) {
-                                        g_free (tmp->data);
-                                        tmp = g_list_next (tmp);
-                                        };
-                                  DEBUG_MSG("free near_misses_list elements done\n");
-                                  g_list_free(near_misses_list);
-                                  DEBUG_MSG("free near_misses_list done\n");
-                                 }; /*end if */
-  g_free(result);
-  DEBUG_MSG("word checked\n");
-} /* end check_word */
 
-/* update slider if and only if value has changed
-		newvalue = int(100.0*newval);
-		if(newvalue!=oldval) {
-			oldval = newvalue;
-			fl_set_slider_value(fd_form_spell_check->slider, oldval);
-		} */
+ /* do a clean up */
+  switch (result->flag) {
+  case ISP_MISSED:   DEBUG_MSG("free & clean near_misses_lists ...\n");
+                     gtk_clist_clear(GTK_CLIST(spc_gui.near_misses_clist));
+                     for(i=0;i<result->count;i++)
+                      g_free(result->misses[i]);
+                     DEBUG_MSG("free & cleaning near_misses_list done\n");
+
+  case ISP_UNKNOWN:  gtk_entry_set_text(GTK_ENTRY(spc_gui.word_entry),"");
+                     gtk_entry_set_text (GTK_ENTRY(spc_gui.replace_entry),"");
+                     DEBUG_MSG("word & replace entries cleaned\n");
+                     }; /*end switch */
+  g_free(result);
+#ifdef DEBUG
+  if (spc_message==SPC_CLOSE)
+    DEBUG_MSG("Passing SPC_CLOSE message\n");
+#endif
+  DEBUG_MSG("word checked\n");
+} /* end check_word  */
+
+
+inline static void update_progress_bar(void)
+{
+ gfloat percentage;
+ gfloat current_position;
+ gfloat text_length;
+ current_position = (gfloat) gtk_text_get_point(GTK_TEXT(CURRENT_DOCUMENT_TEXT));
+ text_length = (gfloat) gtk_text_get_length(GTK_TEXT(CURRENT_DOCUMENT_TEXT));
+ /* percentage is betweeen 0 and 1 */
+ percentage = current_position / text_length;
+ DEBUG_MSG ("percentage done = %f\n",percentage);
+ gtk_progress_bar_update(GTK_PROGRESS_BAR(spc_gui.progress_bar),percentage);
+};
+
+void ispell_terminate()
+{
+ if (isp_pid != -1) /* checks if ispell is running */
+ {
+   fputs("#\n", out); /* Save personal dictionary */
+	fflush(out);
+	fclose(out);
+   kill(isp_pid,SIGTERM);
+   isp_pid = -1; /* ispell is not running.FIXME: Can we check this ? */
+ }
+}
 
 /* Perform an ispell session */
-gboolean run_spell_checker (const gchar* lang)
+gboolean run_spell_checker (void)
 {
 	Tword *word;
-	gint oldval; /* , newvalue; */
-	gfloat newval;
    unsigned int word_count = 0;
-   oldval = 0;  /* used for updating slider only when needed */
-	newval = 0.0;
-
    gtk_text_freeze(GTK_TEXT(CURRENT_DOCUMENT_TEXT));
-    
      /* create ispell process */
 	
     create_ispell_pipe();
     gtk_text_set_point(GTK_TEXT(CURRENT_DOCUMENT_TEXT),0);
+    end_of_text = 0; /* needed for get_next_word to work properly */
 
     if (isp_pid == -1) {
 		                 error_dialog(_("Bluefish Spell Checker Error"),
@@ -570,31 +711,42 @@ gboolean run_spell_checker (const gchar* lang)
 			              "Check /usr/lib/ispell or set another\n"
 			              "dictionary in the Spellchecker Options menu."));
 		                 fclose(out);
-		                 return 1;
+		                 return TRUE;
                      }; /*end if*/
 
+   gtk_grab_add(spc_gui.window); /* This window becames modal */
+
 	/* Put ispell in terse mode to improve speed */
-	/* ispell_terse_mode();*/
+	ispell_terse_mode();
 	do
    {
       word = get_next_word();
+      if (!is_empty_word(word))
+      {
       word_count++;
+      spc_message = SPC_NONE;
       check_word(word);
-      /* FIXME: update_slider() */
       free_word (word);
-      if(spc_message==SPC_DONE)
+      update_progress_bar();
+      if (spc_message==SPC_CLOSE)
                       {
-		                ispell_terminate();
-                      DEBUG_MSG("Ispell died\n");
+                      DEBUG_MSG("Close message received\n");
                       break;
                       };
+     };
    } while (!end_of_text);
    DEBUG_MSG("Spell checking done\n");
-
   gtk_text_thaw(GTK_TEXT(CURRENT_DOCUMENT_TEXT));
-  destroy_spc_window();
+  ispell_terminate();
+  DEBUG_MSG("Removing grab\n");
+  gtk_grab_remove(spc_gui.window);
+  DEBUG_MSG("Removing status \n");
+  statusbar_remove(GINT_TO_POINTER(spc_gui.status_bar_count));
+  DEBUG_MSG("Destroying window\n");
+  gtk_widget_destroy(spc_gui.window);
 
-return (spc_message != SPC_DONE);
+  return (spc_message != SPC_CLOSE);
 } /* end run_spell_checker */
 
-#endif
+
+
