@@ -23,11 +23,16 @@
 
 #include <glib.h>
 
+void estrl_dialog(GList **which_list, gchar *title, gint what_list
+				, gint column_num, gchar **column_titles, void *post_dialog_func);
+
 gint free_stringlist(GList * which_list);
 GList *get_stringlist(gchar * filename, GList * which_list);
 gint put_stringlist(gchar * filename, GList * which_list);
-GList *edit_stringlist(GList * which_list, gchar * title, gint what_list);
-
+gint count_array(gchar **array);
+gint free_arraylist(GList * which_list);
+gchar **duplicate_stringarray(gchar **array);
+GList *duplicate_arraylist(GList *arraylist);
 /* adds a string to a stringlist if it is not yet in there */
 GList *add_to_stringlist(GList * which_list, gchar * string);
 
