@@ -33,10 +33,15 @@
 #include "pixmaps.h"
 #include "toolbars.h"
 #include "callbacks.h"
+#include "coloursel.h"  /* sel_colour_cb() */
+#include "document.h"
 #include "html.h"
+#include "html_form.h"
+#include "html_table.h"
 #include "html2.h"
 #include "wml.h"
 #include "interface.h"
+#include "javascript.h"
 #include "images.h"
 #include "wizards.h"
 #include "ref_dialog.h"
@@ -65,12 +70,6 @@ typedef struct {
 	cust_main_menu_struct *main_entry;
 } cust_menu_struct;
 
-/* 
- * these widgets are NULL if the bars are not initialised yet,
- * they're !NULL after initialisation 
- */
-GtkWidget *main_widget = NULL, *cust_widget = NULL, *html_widget = NULL;
-
 
 GList *entry_list_list;
 GtkWidget *entry1, *entry2, *entry3, *entry4;
@@ -79,16 +78,8 @@ cust_main_menu_struct *last_main_entry = NULL; */
 
 
 /***************************************************************************/
-void toggle_html_toolbar_lcb(GtkWidget * w, gpointer data);
-void toggle_main_toolbar_lcb(GtkWidget * w, gpointer data);
-void hide_main_toolbar_cb(GtkWidget * w, gpointer data);
-void hide_html_toolbar_cb(GtkWidget * w, gpointer data);
-void show_main_toolbar_cb(GtkWidget * w, gpointer data);
-void show_html_toolbar_cb(GtkWidget * w, gpointer data);
-void make_main_toolbar(void);
-void make_html_toolbar(void);
-void flush_toolbar(gchar * labeltext, GtkWidget * toolbar, GtkWidget * notebook);
-GtkWidget *new_pixmap(int type, GdkWindow * window, GdkColor * background);
+static void flush_toolbar(gchar * labeltext, GtkWidget * toolbar, GtkWidget * notebook);
+/* GtkWidget *new_pixmap(int type, GdkWindow * window, GdkColor * background); */
 
 
 /***************************************************************************/
@@ -97,55 +88,11 @@ GtkWidget *new_pixmap(int type, GdkWindow * window, GdkColor * background);
 /***************************************************************************/
 /* takes care of inserting the special characters */
 
-void toggle_html_toolbar_lcb(GtkWidget * w, gpointer data)
-{
-	if (GTK_WIDGET_VISIBLE(html_handle_box)) {
-		hide_html_toolbar_cb(NULL, NULL);
-	} else {
-		show_html_toolbar_cb(NULL, NULL);
-	}
-}
 
-void toggle_main_toolbar_lcb(GtkWidget * w, gpointer data)
+void make_main_toolbar(GtkWidget *handle_box)
 {
-	if (GTK_WIDGET_VISIBLE(handle_box)) {
-		hide_main_toolbar_cb(NULL, NULL);
-	} else {
-		show_main_toolbar_cb(NULL, NULL);
-	}
-}
-
-void hide_main_toolbar_cb(GtkWidget * w, gpointer data)
-{
-	gtk_widget_hide(handle_box);
-}
-
-void hide_html_toolbar_cb(GtkWidget * w, gpointer data)
-{
-	gtk_widget_hide(html_handle_box);
-}
-
-void show_main_toolbar_cb(GtkWidget * w, gpointer data)
-{
-	if (!main_widget) {
-		make_main_toolbar();
-	}
-	gtk_widget_show(handle_box);
-}
-
-void show_html_toolbar_cb(GtkWidget * w, gpointer data)
-{
-	if (!html_widget) {
-		make_html_toolbar();
-	}
-	gtk_widget_show(html_handle_box);
-}
-
-void make_main_toolbar(void)
-{
-
+	GtkWidget *main_widget = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
 	DEBUG_MSG("make_main_toolbar, started\n");
-	main_widget = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
 	gtk_container_add(GTK_CONTAINER(handle_box), main_widget);
 	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("New"), "",
 							new_pixmap(000, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(file_new_cb), NULL);
@@ -178,16 +125,21 @@ void make_main_toolbar(void)
 	gtk_toolbar_append_space(GTK_TOOLBAR(main_widget));
 	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Configure"),
 							"", new_pixmap(014, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(configure_cb), NULL);
-	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Print"), "", new_pixmap(015, main_v->main_window->window, NULL), NULL, NULL);
+	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Print"), "",
+							new_pixmap(015, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(file_print_cb), NULL);
 #ifdef WITH_SPC	
 	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Spellcheck"),
 							"", new_pixmap(016, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(spell_check_cb), NULL);
 #endif	
+	gtk_toolbar_append_space(GTK_TOOLBAR(main_widget));
+	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL,
+							_("View in Netscape"), "",
+							new_pixmap(170, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(view_in_netscape_cb), NULL);
 	gtk_widget_show(main_widget);
 	DEBUG_MSG("make_main_widget, end\n");
 }
 
-void make_html_toolbar(void)
+void make_html_toolbar(GtkWidget *html_handle_box)
 {
 
 /*
@@ -196,7 +148,10 @@ void make_html_toolbar(void)
  *
  */
 
-	GtkWidget *html_toolbar;
+	GtkWidget *html_toolbar, *html_widget;
+	GtkWidget *menu_bar;	/* for construction of a dropdown menu */
+	GtkWidget *menu_item;
+	GtkWidget *sub_menu;
 
 	DEBUG_MSG("make_html_toolbar, started\n");
 	html_widget = gtk_notebook_new();
@@ -208,22 +163,88 @@ void make_html_toolbar(void)
 
 	html_toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL,
-							_("QuickStart..."), "", new_pixmap(100, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(quickstart), NULL);
+							_("QuickStart..."), "", new_pixmap(100, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(quickstart_cb), NULL);
+/* not used anymore
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("DTD..."),
-							"", new_pixmap(185, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(dtd_cb), NULL);
+							"", new_pixmap(185, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(quickstart_cb), NULL);
+
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Head..."),
-							"", new_pixmap(186, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(head_cb), NULL);
+							"", new_pixmap(186, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(quickstart_cb), NULL);
+*/
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Body..."),
 							"", new_pixmap(101, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(body_cb), NULL);
 	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Bold"), "",
+	if (main_v->props.format_by_context) {
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Strong"), "",
+							new_pixmap(102, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 16);
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Emphasis"), "",
+							new_pixmap(103, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 17);	
+	} else {
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Bold"), "",
 							new_pixmap(102, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 1);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Italic"), "",
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Italic"), "",
 							new_pixmap(103, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 2);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Underline"),
-							"", new_pixmap(104, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 3);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Strikeout"),
-							"", new_pixmap(105, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 4);
+	}
+	if(main_v->props.allow_dep){
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Underline"), "", new_pixmap(104,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 3);
+		if(main_v->props.allow_h4)gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Strikeout"), "", new_pixmap(105,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 4);
+	}
+	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
+
+	/* headings dropdown menu */
+	sub_menu = gtk_menu_new();
+	/*H1*/
+	menu_item = gtk_menu_item_new();
+	gtk_container_add(GTK_CONTAINER(menu_item), 
+				new_pixmap(127, main_v->main_window->window, NULL));
+	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 18);
+	gtk_menu_append(GTK_MENU(sub_menu), menu_item);
+	/*H2*/
+	menu_item = gtk_menu_item_new();
+	gtk_container_add(GTK_CONTAINER(menu_item), 
+				new_pixmap(128, main_v->main_window->window, NULL));
+	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 19);
+	gtk_menu_append(GTK_MENU(sub_menu), menu_item);
+	/*H3*/
+	menu_item = gtk_menu_item_new();
+	gtk_container_add(GTK_CONTAINER(menu_item), 
+				new_pixmap(129, main_v->main_window->window, NULL));
+	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 20);
+	gtk_menu_append(GTK_MENU(sub_menu), menu_item);
+	/*H4*/
+	menu_item = gtk_menu_item_new();
+	gtk_container_add(GTK_CONTAINER(menu_item), 
+				new_pixmap(130, main_v->main_window->window, NULL));
+	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 21);
+	gtk_menu_append(GTK_MENU(sub_menu), menu_item);
+	/*H5*/
+	menu_item = gtk_menu_item_new();
+	gtk_container_add(GTK_CONTAINER(menu_item), 
+				new_pixmap(131, main_v->main_window->window, NULL));
+	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 22);
+	gtk_menu_append(GTK_MENU(sub_menu), menu_item);
+	/*H6*/
+	menu_item = gtk_menu_item_new();
+	gtk_container_add(GTK_CONTAINER(menu_item), 
+				new_pixmap(132, main_v->main_window->window, NULL));
+	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 23);
+	gtk_menu_append(GTK_MENU(sub_menu), menu_item);
+
+	menu_bar = gtk_menu_bar_new();
+	menu_item = gtk_menu_item_new();
+	gtk_container_add(GTK_CONTAINER(menu_item), 
+				new_pixmap(191, main_v->main_window->window, NULL));
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), sub_menu);
+	gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), menu_item);
+	gtk_widget_show_all(GTK_WIDGET(menu_bar));
+	gtk_toolbar_append_element(GTK_TOOLBAR(html_toolbar),
+							GTK_TOOLBAR_CHILD_WIDGET, menu_bar,
+							NULL, _("Headings"), NULL,
+							new_pixmap(127, main_v->main_window->window, NULL),
+							NULL, NULL);
+
 	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Paragraph"),
 							"", new_pixmap(106, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 5);
@@ -242,9 +263,11 @@ void make_html_toolbar(void)
 							"", new_pixmap(112, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(quickrule_cb), NULL);
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Image..."),
 							"", new_pixmap(113, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(image_insert_dialog_cb), NULL);
+#if HAVE_LIBGDK_IMLIB
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL,
 							"Thumbnail...",
 							"", new_pixmap(125, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(image_thumbnail_dialog_cb), NULL);
+#endif
 	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Center"), "",
 							new_pixmap(114, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 8);
@@ -263,29 +286,88 @@ void make_html_toolbar(void)
 
 
 	html_toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Font..."),
-							"", new_pixmap(118, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(fontdialog_cb), NULL);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL,
-							_("Base Font Size..."), "", new_pixmap(119,
-																   main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(basefont_cb), NULL);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL,
-							_("Font Size +1"),
-							"", new_pixmap(120, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 11);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL,
-							_("Font Size -1"),
-							"", new_pixmap(121, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 12);
-	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
+	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Font..."), "", new_pixmap(118,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(fontdialog_cb), NULL);
+	if(main_v->props.allow_dep){
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Base Font Size..."), "", new_pixmap(119,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(basefont_cb), NULL);
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Font Size +1"), "", new_pixmap(120,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 11);
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Font Size -1"), "", new_pixmap(121,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 12);
+		gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
+	}
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL,
 							_("Preformatted Text"), "", new_pixmap(122,
 																   main_v->main_window->window,
 																   NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 13);
 	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Subscript"),
-							"", new_pixmap(123, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 14);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL,
-							_("Superscript"),
-							"", new_pixmap(124, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 15);
-	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
+
+	/* headings dropdown menu */
+	sub_menu = gtk_menu_new();
+	/*abbr*/
+	menu_item = gtk_menu_item_new();
+	gtk_container_add(GTK_CONTAINER(menu_item), 
+				new_pixmap(425, main_v->main_window->window, NULL));
+	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 54);
+	gtk_menu_append(GTK_MENU(sub_menu), menu_item);
+	/*acronym*/
+	menu_item = gtk_menu_item_new();
+	gtk_container_add(GTK_CONTAINER(menu_item), 
+				new_pixmap(418, main_v->main_window->window, NULL));
+	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 55);
+	gtk_menu_append(GTK_MENU(sub_menu), menu_item);
+	/*cite*/
+	menu_item = gtk_menu_item_new();
+	gtk_container_add(GTK_CONTAINER(menu_item), 
+				new_pixmap(419, main_v->main_window->window, NULL));
+	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 53);
+	gtk_menu_append(GTK_MENU(sub_menu), menu_item);
+	/* code */
+	menu_item = gtk_menu_item_new();
+	gtk_container_add(GTK_CONTAINER(menu_item), 
+				new_pixmap(420, main_v->main_window->window, NULL));
+	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 49);
+	gtk_menu_append(GTK_MENU(sub_menu), menu_item);
+	/*dfn*/
+	menu_item = gtk_menu_item_new();
+	gtk_container_add(GTK_CONTAINER(menu_item), 
+				new_pixmap(422, main_v->main_window->window, NULL));
+	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 48);
+	gtk_menu_append(GTK_MENU(sub_menu), menu_item);
+	/*samp*/
+	menu_item = gtk_menu_item_new();
+	gtk_container_add(GTK_CONTAINER(menu_item), 
+				new_pixmap(423, main_v->main_window->window, NULL));
+	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 50);
+	gtk_menu_append(GTK_MENU(sub_menu), menu_item);
+	/*var*/
+	menu_item = gtk_menu_item_new();
+	gtk_container_add(GTK_CONTAINER(menu_item), 
+				new_pixmap(424, main_v->main_window->window, NULL));
+	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 52);
+	gtk_menu_append(GTK_MENU(sub_menu), menu_item);
+
+	menu_bar = gtk_menu_bar_new();
+	menu_item = gtk_menu_item_new();
+	gtk_container_add(GTK_CONTAINER(menu_item), 
+				new_pixmap(420, main_v->main_window->window, NULL));
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), sub_menu);
+	gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), menu_item);
+	gtk_widget_show_all(GTK_WIDGET(menu_bar));
+	gtk_toolbar_append_element(GTK_TOOLBAR(html_toolbar),
+							GTK_TOOLBAR_CHILD_WIDGET, menu_bar,
+							NULL, _("Context formatting"), NULL,
+							new_pixmap(420, main_v->main_window->window, NULL),
+							NULL, NULL);
+
+	if(main_v->props.allow_dep){
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Subscript"), "", new_pixmap(123,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 14);
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Superscript"), "", new_pixmap(124,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 15);
+		gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
+	}
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Strong"), "",
 							new_pixmap(102, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 16);
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Emphasis"),
@@ -341,39 +423,35 @@ void make_html_toolbar(void)
 
 	flush_toolbar(_(" Tables "), html_toolbar, html_widget);
 
+	if(main_v->props.allow_frames){
+		html_toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Frame Wizard..."), "", new_pixmap(143,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(framewizard), NULL);
+		gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Frameset..."), "", new_pixmap(144,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(framesetdialog_cb), NULL);
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Frame..."),	"", new_pixmap(145,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(framedialog_cb), NULL);
+		gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Frameset"),	"", new_pixmap(146,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 29);
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Frame"), "", new_pixmap(147,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 30);
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Noframes"), "", new_pixmap(148,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 31);
+		gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Base Target"), "", new_pixmap(149,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 32);
 
-	html_toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL,
-							_("Frame Wizard..."),
-							"", new_pixmap(143, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(framewizard), NULL);
-	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL,
-							_("Frameset..."),
-							"", new_pixmap(144, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(framesetdialog_cb), NULL);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Frame..."),
-							"", new_pixmap(145, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(framedialog_cb), NULL);
-	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Frameset"),
-							"", new_pixmap(146, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 29);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Frame"), "",
-							new_pixmap(147, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 30);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Noframes"),
-							"", new_pixmap(148, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 31);
-	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL,
-							_("Base Target"),
-							"", new_pixmap(149, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 32);
-
-	flush_toolbar(_(" Frames "), html_toolbar, html_widget);
+		flush_toolbar(_(" Frames "), html_toolbar, html_widget);
+	}
 
 	html_toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Form..."),
 							"", new_pixmap(150, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(formdialog_cb), NULL);
 	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Submit..."),
-							"", new_pixmap(151, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(submitdialog_cb), NULL);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Reset..."),
-							"", new_pixmap(152, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(resetdialog_cb), NULL);
+	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Input button..."),
+							"", new_pixmap(151, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(buttondialog_cb), NULL);
 	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Text..."),
 							"", new_pixmap(153, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(textdialog_cb), NULL);
@@ -392,6 +470,12 @@ void make_html_toolbar(void)
 	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Select..."),
 							"", new_pixmap(158, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(selectdialog_cb), NULL);
+	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
+	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Option..."),
+							"", new_pixmap(417, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(optiondialog_cb), NULL);
+	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Option group..."),
+							"", new_pixmap(300, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(optgroupdialog_cb), NULL);
+
 	flush_toolbar(_(" Forms "), html_toolbar, html_widget);
 
 	html_toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
@@ -421,32 +505,53 @@ void make_html_toolbar(void)
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Definition"),
 							"", new_pixmap(165, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *)
 							(gpointer *) 38);
-	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Menu"), "",
-							new_pixmap(166, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *)
-							(gpointer *) 39);
+	if(main_v->props.allow_dep){
+		gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Menu"), "",	new_pixmap(166,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *)(gpointer *) 39);
+	}
 
 	flush_toolbar(_(" Lists "), html_toolbar, html_widget);
 
 	/* CSS toolbar */
-	html_toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Insert style in stylesheet..."),
-							"", new_pixmap(180, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(insert_in_stylesheet), NULL);
-	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Span..."),
-							"", new_pixmap(181, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(insert_span), NULL);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Div..."),
-							"", new_pixmap(182, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(insert_div), NULL);
-	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Add class to tag..."),
-							"", new_pixmap(183, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(insert_class), NULL);
-	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Style"),
-							"", new_pixmap(184, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 42);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Link to stylesheet"),
-							"", new_pixmap(189, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 46);
+	if(main_v->props.allow_css){
+		html_toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Create stylesheet..."), "", new_pixmap(180,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(new_css_dialog), NULL);
+		gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Span..."), "", new_pixmap(181,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(span_dialog), NULL);
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Div..."), "", new_pixmap(182,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(div_dialog), NULL);
+		gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
+#ifdef OLD_CSS_DIALOG
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Add class to tag..."), "", new_pixmap(183,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(insert_class), NULL);
+#endif /* OLD_CSS_DIALOG */
+		gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Style"), "", new_pixmap(184,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 42);
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Link to stylesheet"), "", new_pixmap(189,
+			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(link_cb), NULL);
 
-	flush_toolbar(_(" CSS "), html_toolbar, html_widget);
+		flush_toolbar(_(" CSS "), html_toolbar, html_widget);
+	}
+
+  
+  /* javascript toolbar */
+	
+	html_toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
+	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Javascript for Mouseover Effects"),
+							"", new_pixmap(500, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(mouseover_script_cb), NULL);
+	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
+	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Javascript to work around Resize Bug in Netscape 4"),
+							"", new_pixmap(501, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(ns47_bugfix_script_cb), NULL);
+	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
+	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Empty javascript block"),
+							"", new_pixmap(188, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 44);
+	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Link to javascript"),
+							"", new_pixmap(187, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(script_cb), NULL);
+	flush_toolbar(_(" Javascript "), html_toolbar, html_widget);
 
 	/* WML toolbar */
 	html_toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
@@ -499,10 +604,6 @@ void make_html_toolbar(void)
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Embed..."),
 							"", new_pixmap(169, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(embed_cb), NULL);
 	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL,
-							_("View in Netscape"), "",
-							new_pixmap(170, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(view_in_netscape_cb), NULL);
-	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL,
 							_("Select Colour..."), "",
 							new_pixmap(171, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(sel_colour_cb), NULL);
@@ -511,7 +612,7 @@ void make_html_toolbar(void)
 							new_pixmap(172, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(insert_time_cb), NULL);
 	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL,
-							_("PHP3 reference..."), "",
+							_("PHP reference..."), "",
 							new_pixmap(173, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(php3_dialog), NULL);
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL,
 							_("SSI reference..."), "",
@@ -519,11 +620,11 @@ void make_html_toolbar(void)
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL,
 							_("RXML reference..."), "",
 							new_pixmap(175, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(rxml_dialog), NULL);
-	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Javascript"),
-							"", new_pixmap(188, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 44);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Link to javascript"),
-							"", new_pixmap(187, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 45);
+	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL,
+							_("PHP4 reference..."), "",
+							new_pixmap(173, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(php4_dialog), NULL);
+	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Insert Generator META-Tag"),
+							"", new_pixmap(190, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 47);
 	flush_toolbar(_(" Other "), html_toolbar, html_widget);
 
 	gtk_widget_show(html_widget);
@@ -531,7 +632,7 @@ void make_html_toolbar(void)
 }
 
 
-void flush_toolbar(gchar * labeltext, GtkWidget * toolbar, GtkWidget * notebook)
+static void flush_toolbar(gchar * labeltext, GtkWidget * toolbar, GtkWidget * notebook)
 {
 	GtkWidget *html_label;
 
