@@ -25,12 +25,7 @@
 #include <gtk/gtkwidget.h>
 #include <gdk/gdktypes.h>
 
-void show_html_toolbar_cb(GtkWidget * w, gpointer data);
-void show_main_toolbar_cb(GtkWidget * w, gpointer data);
-void hide_html_toolbar_cb(GtkWidget * w, gpointer data);
-void hide_main_toolbar_cb(GtkWidget * w, gpointer data);
-GtkWidget *new_pixmap(int type, GdkWindow * window, GdkColor * background);
-void toggle_main_toolbar_lcb(GtkWidget * w, gpointer data);
-void toggle_html_toolbar_lcb(GtkWidget * w, gpointer data);
+void make_main_toolbar(GtkWidget *handle_box);
+void make_html_toolbar(GtkWidget *html_handle_box);
 
 #endif							/* __TOOLBARS_H_ */
